(*----------------------------------------------------------------------------*
 *  Direct3D sample from DirectX 9.0 SDK Summer 2003                          *
 *  Delphi adaptation by Alexey Barkovoy (e-mail: clootie@ixbt.com)           *
 *                                                                            *
 *  Delphi versions 5-7 are supported                                         *
 *                                                                            *
 *  Modified: 02-Feb-2004                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru/delphi                                         *
 *----------------------------------------------------------------------------*)
//-----------------------------------------------------------------------------
// File: SHPRTPixel.h
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// File: SHPRTPixel.cpp
//
// Desc: This sample demonstrates how use D3DXSHPRTSimulationTex(), a per texel 
//       precomputed radiance transfer (PRT) simulator that uses low-order 
//       spherical harmonics (SH) and records the results to a file. The sample 
//       then demonstrates how compress the results with principal 
//       component analysis (PCA) and views the compressed results with arbitrary 
//       lighting in real time with a ps_2_0 pixel shader
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------

unit SHPRTPixelUnit;

interface

uses
  Windows, Messages, CommCtrl, SyncObjs,
  Direct3D9, D3DX9, dxerr9,
  D3DApp, D3DFont, D3DUtil, DXUtil,
  Resource;

// C++Builder compile hack
{$NOINCLUDE Resource}
{$HPPEMIT '#include "Resource.hpp"'}

{.$DEFINE DEBUG_VS}   // Uncomment this line to debug vertex shaders
{.$DEFINE DEBUG_PS}   // Uncomment this line to debug pixel shaders



//-----------------------------------------------------------------------------
// Globals variables and definitions
//-----------------------------------------------------------------------------
const
  MAX_PCA_VECTORS      = 24;
  MAX_LIGHTS           = 3;

const
  g_fSqrtPI = 1.7724538510212332182745076025231; // Sqrt(D3DX_PI);
//  g_hInst: THandle {HINSTANCE} = 0;

type
  // Struct to store material params
  PPredefinedMaterials = ^TPredefinedMaterials;
  TPredefinedMaterials = record
    strName: {const} PChar;
    fScatteringCoefficient: TD3DColorValue;
    fAbsorptionCoefficient: TD3DColorValue;
    fDiffuseReflectance: TD3DColorValue;
    fRelativeIndexOfRefraction: Single;
  end;

const
  // Subsurface scattering parameters from:
  // "A Practical Model for Subsurface Light Transport",
  // Henrik Wann Jensen, Steve R. Marschner, Marc Levoy, Pat Hanrahan.
  // SIGGRAPH 2001
  g_aPredefinedMaterials: array[0..13] of TPredefinedMaterials =
  (
           // name             scattering (R/G/B/A)                                     absorption (R/G/B/A)                                          reflectance (R/G/B/A)                               index of refraction
    (strName:'Default';    fScatteringCoefficient:(r:2.00; g:2.00; b:2.00; a:1.0); fAbsorptionCoefficient:(r:0.0030; g:0.0030; b:0.0460; a:1.0); fDiffuseReflectance:(r:1.00; g:1.00; b:1.00; a:1.0); fRelativeIndexOfRefraction: 1.3),
    (strName:'Apple';      fScatteringCoefficient:(r:2.29; g:2.39; b:1.97; a:1.0); fAbsorptionCoefficient:(r:0.0030; g:0.0030; b:0.0460; a:1.0); fDiffuseReflectance:(r:0.85; g:0.84; b:0.53; a:1.0); fRelativeIndexOfRefraction: 1.3),
    (strName:'Chicken1';   fScatteringCoefficient:(r:0.15; g:0.21; b:0.38; a:1.0); fAbsorptionCoefficient:(r:0.0150; g:0.0770; b:0.1900; a:1.0); fDiffuseReflectance:(r:0.31; g:0.15; b:0.10; a:1.0); fRelativeIndexOfRefraction: 1.3),
    (strName:'Chicken2';   fScatteringCoefficient:(r:0.19; g:0.25; b:0.32; a:1.0); fAbsorptionCoefficient:(r:0.0180; g:0.0880; b:0.2000; a:1.0); fDiffuseReflectance:(r:0.32; g:0.16; b:0.10; a:1.0); fRelativeIndexOfRefraction: 1.3),
    (strName:'Cream';      fScatteringCoefficient:(r:7.38; g:5.47; b:3.15; a:1.0); fAbsorptionCoefficient:(r:0.0002; g:0.0028; b:0.0163; a:1.0); fDiffuseReflectance:(r:0.98; g:0.90; b:0.73; a:1.0); fRelativeIndexOfRefraction: 1.3),
    (strName:'Ketchup';    fScatteringCoefficient:(r:0.18; g:0.07; b:0.03; a:1.0); fAbsorptionCoefficient:(r:0.0610; g:0.9700; b:1.4500; a:1.0); fDiffuseReflectance:(r:0.16; g:0.01; b:0.00; a:1.0); fRelativeIndexOfRefraction: 1.3),
    (strName:'Marble';     fScatteringCoefficient:(r:2.19; g:2.62; b:3.00; a:1.0); fAbsorptionCoefficient:(r:0.0021; g:0.0041; b:0.0071; a:1.0); fDiffuseReflectance:(r:0.83; g:0.79; b:0.75; a:1.0); fRelativeIndexOfRefraction: 1.5),
    (strName:'Potato';     fScatteringCoefficient:(r:0.68; g:0.70; b:0.55; a:1.0); fAbsorptionCoefficient:(r:0.0024; g:0.0090; b:0.1200; a:1.0); fDiffuseReflectance:(r:0.77; g:0.62; b:0.21; a:1.0); fRelativeIndexOfRefraction: 1.3),
    (strName:'Skimmilk';   fScatteringCoefficient:(r:0.70; g:1.22; b:1.90; a:1.0); fAbsorptionCoefficient:(r:0.0014; g:0.0025; b:0.0142; a:1.0); fDiffuseReflectance:(r:0.81; g:0.81; b:0.69; a:1.0); fRelativeIndexOfRefraction: 1.3),
    (strName:'Skin1';      fScatteringCoefficient:(r:0.74; g:0.88; b:1.01; a:1.0); fAbsorptionCoefficient:(r:0.0320; g:0.1700; b:0.4800; a:1.0); fDiffuseReflectance:(r:0.44; g:0.22; b:0.13; a:1.0); fRelativeIndexOfRefraction: 1.3),
    (strName:'Skin2';      fScatteringCoefficient:(r:1.09; g:1.59; b:1.79; a:1.0); fAbsorptionCoefficient:(r:0.0130; g:0.0700; b:0.1450; a:1.0); fDiffuseReflectance:(r:0.63; g:0.44; b:0.34; a:1.0); fRelativeIndexOfRefraction: 1.3),
    (strName:'Spectralon';fScatteringCoefficient:(r:11.60;g:20.40;b:14.90; a:1.0); fAbsorptionCoefficient:(r:0.0000; g:0.0000; b:0.0000; a:1.0); fDiffuseReflectance:(r:1.00; g:1.00; b:1.00; a:1.0); fRelativeIndexOfRefraction: 1.3),
    (strName:'Wholemilk';  fScatteringCoefficient:(r:2.55; g:3.21; b:3.77; a:1.0); fAbsorptionCoefficient:(r:0.0011; g:0.0024; b:0.0140; a:1.0); fDiffuseReflectance:(r:0.91; g:0.88; b:0.76; a:1.0); fRelativeIndexOfRefraction: 1.3),
    (strName:'Custom';     fScatteringCoefficient:(r:0.00; g:0.00; b:0.00; a:1.0); fAbsorptionCoefficient:(r:0.0000; g:0.0000; b:0.0000; a:1.0); fDiffuseReflectance:(r:0.00; g:0.00; b:0.00; a:1.0); fRelativeIndexOfRefraction: 0.0)
  );

  g_aPredefinedMaterialsSize = SizeOf(g_aPredefinedMaterials) div SizeOf(g_aPredefinedMaterials[0]);

type
  PSingleArray = ^TSingleArray;
  TSingleArray = array[0..MaxInt div SizeOf(Single) - 1] of Single;
  
type
  //-----------------------------------------------------------------------------
  // Name: class CMyD3DApplication
  // Desc: Main class to run this application. Most functionality is inherited
  //       from the CD3DApplication base class.
  //-----------------------------------------------------------------------------
  CMyD3DApplication = class(CD3DApplication)
  private
    m_pFont: ID3DXFont;
    m_bShowHelp: Boolean;
    m_Camera: CModelViewerCamera;
    m_pSHPRTEffect: ID3DXEffect;
    m_pSimpleLightingEffect: ID3DXEffect;
    m_cs: TRTLCriticalSection;
    m_bWireFrame: Boolean;
    m_bSHPRTReadyToRender: Boolean;

    m_pMesh: ID3DXMesh;
    m_pSHPRTBuffer: ID3DXBuffer;
    m_pPCAWeightTexture: array[0..MAX_PCA_VECTORS div 4] of IDirect3DTexture9;
    m_fObjectRadius: Single;
    m_vObjectCenter: TD3DXVector3;

    m_mWorld: TD3DXMatrixA16;
    m_mView: TD3DXMatrixA16;
    m_mProj: TD3DXMatrixA16;

    m_nTechnique: Integer;
    m_pSphere: ID3DXMesh;

    m_LightArcBall:      array[0..MAX_LIGHTS-1] of CD3DArcBall;
    m_vDefaultLightDir:  array[0..MAX_LIGHTS-1] of TD3DXVector3;
    m_vLightDir:         array[0..MAX_LIGHTS-1] of TD3DXVector3;
    m_mLightRot:         array[0..MAX_LIGHTS-1] of TD3DXMatrixA16;
    m_mLightRotSnapshot: array[0..MAX_LIGHTS-1] of TD3DXMatrixA16;
    m_fLightScale: Single;
    m_nNumActiveLights: Integer;
    m_nActiveLight: Integer;

    m_bPRTSimulationComplete: Boolean;
    m_bPRTSimulationFailed: Boolean;
    m_hPRTSimulationThreadId: THandle;
    m_dwPRTSimulationThreadId: DWORD;
    m_bStopSimulator: Boolean; //todo: WAS "float" - fill BUG report
    m_fPercentDone: Single;

    // Simulation options
    m_strInitialDir: array[0..MAX_PATH-1] of Char;
    m_strInputMesh: array[0..MAX_PATH-1] of Char;
    m_dwOrder:               DWORD;
    m_dwTextureSize:         DWORD;
    m_dwNumRays:             DWORD;
    m_dwNumBounces:          DWORD;
    m_dwNumPCAVectors:       DWORD;
    m_fmtTexture:            TD3DFormat;
    m_bSubsurfaceScattering: Boolean;
    m_fLengthScale:          Single;
    m_bSpectral:             Boolean;
    m_dwPredefinedMatIndex:  DWORD;
    m_DiffuseReflectance:    TD3DXColor;
    m_Absoption:             TD3DXColor;
    m_ReducedScattering:     TD3DXColor;
    m_fRelativeIndexOfRefraction: Single;
    m_strResultsFile: array[0..MAX_PATH-1] of Char;

    // The basis buffer is a large array of floats where
    // Call D3DXSHPRTCompExtractBasis() to extract the basis
    // for every cluster.  The basis for a cluster is an array of
    // (NumPCAVectors+1)*(NumChannels*Order^2) floats.
    // The "1+" is for the cluster mean.
    m_aClusterBases: PSingleArray;

    // m_fClusteredPCA stores the incident radiance dotted with the transfer function.
    // Each cluster has an array of floats which is the size of
    // 4+MAX_NUM_CHANNELS*NUM_PCA_VECTORS. This number comes from: there can
    // be up to 3 channels (R,G,B), and each channel can
    // have up to NUM_PCA_VECTORS of PCA vectors.  Each cluster also has
    // a mean PCA vector which is described with 4 floats (and hence the +4).
    m_aClusteredPCA: PSingleArray;

  protected
    function OneTimeSceneInit: HRESULT; override;
    function InitDeviceObjects: HRESULT; override;
    function RestoreDeviceObjects: HRESULT; override;
    function InvalidateDeviceObjects: HRESULT; override;
    function DeleteDeviceObjects: HRESULT; override;
    function FinalCleanup: HRESULT; override;
    function Render: HRESULT; override;
    function FrameMove: HRESULT; override;
    function ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
      adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT; override;

    function DoStartupDlg: HRESULT;
    function LoadMeshAndResults: HRESULT;
    function DoCompressDlg: HRESULT;
    function LoadMesh(strFileName: PChar; out ppMesh: ID3DXMesh): HRESULT;
    function UpdateUI: HRESULT;
    function UpdateLightDir: HRESULT;

    function RenderLightSphere(lightDir: TD3DXVector3; sphereColor: TD3DXColor): HRESULT;
    function RenderMeshWithSHPRT: HRESULT;
    function RenderMeshWithSimpleLighting: HRESULT;
    function RenderText: HRESULT;

    function LaunchPRTSimulation: HRESULT;
    function StopPRTSimulation: HRESULT;
    function CompressData: HRESULT;
    // static DWORD WINAPI StaticPRTSimulationThreadProc( LPVOID lpParameter );
    function PRTSimulationThreadProc: DWORD;
    // static HRESULT WINAPI StaticSHPRTSimulatorCB( float fPercentDone );
    function SHPRTSimulatorCB(fPercentDone: Single): HRESULT;

    function SetupTexWithPCAData(pMesh: ID3DXMesh; pPCABuffer: ID3DXBuffer; desc: TD3DXSHPRTBufferDesc; dwNumPCAVectors: DWORD): HRESULT;
    function EvalLightsAndSetConstants: HRESULT;
    function SetShaderConstants(pfRLC, pfGLC, pfBLC: PSingle): HRESULT;
    function LoadSHPRTEffect: HRESULT;

    function SaveSHPRTBufferToFile(pd3dxBuffer: ID3DXBuffer; strFile: PChar): HRESULT;
    function ReadSHPRTBufferFromFile(strFile: PChar; out ppd3dxBuffer: ID3DXBuffer): HRESULT;

    // static INT_PTR CALLBACK StaticStartupDlgProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );
    // static INT_PTR CALLBACK StaticSimulationOptionsDlgProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );
    // static INT_PTR CALLBACK StaticCompressDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam );
    function SimulationOptionsDlgProc(hDlg: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT {INT_PTR}; stdcall;
    function StartupDlgProc(hDlg: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT {INT_PTR}; stdcall;
    function CompressDlgProc(hDlg: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT {INT_PTR}; stdcall;
  public
    constructor Create;
    destructor Destroy; override;
    function MsgProc(hWnd: HWND; uMsg: LongWord; wParam: WPARAM; lParam: LPARAM): LRESULT; override;
  end;

var
  g_d3dApp: CMyD3DApplication;


function StaticPRTSimulationThreadProc(lpParameter: Pointer): DWORD; stdcall;
function StaticSHPRTSimulatorCB(fPercentDone: Single): HRESULT; stdcall;
function StaticStartupDlgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT {INT_PTR}; stdcall;
function StaticCompressDlgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT {INT_PTR}; stdcall;
function StaticSimulationOptionsDlgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT {INT_PTR}; stdcall;


implementation

uses SysUtils, Math, CommDlg, OptionsDlg;

const
  UnitName = 'SHPRTVertexUnit.pas';

{ CMyD3DApplication }

//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor.   Paired with ~CMyD3DApplication()
//       Member variables should be initialized to a known state here.  
//       The application window has not yet been created and no Direct3D device 
//       has been created, so any initialization that depends on a window or 
//       Direct3D should be deferred to a later stage. 
//-----------------------------------------------------------------------------
constructor CMyD3DApplication.Create;
var
  i: Integer;
  hRegKey: HKEY;
  dwResult: DWORD;
  strMedia: array[0..MAX_PATH-1] of Char;
begin
  inherited Create;
  m_Camera:= CModelViewerCamera.Create;
  for i:= 0 to MAX_LIGHTS-1 do
    m_LightArcBall[i]:= CD3DArcBall.Create;

  m_strWindowTitle := 'SHPRTPixel';
  m_d3dEnumeration.AppUsesDepthBuffer := True;
  m_pFont := nil;
  m_bCreateMultithreadDevice := True;
  m_bShowCursorWhenFullscreen := True;
  m_nTechnique := 0;
  m_dwCreationWidth  := 640;
  m_dwCreationHeight := 480;
  m_bWireFrame := False;
  m_fObjectRadius := 1.0;
  m_vObjectCenter := D3DXVector3(0,0,0);

  m_bPRTSimulationComplete := False;
  m_hPRTSimulationThreadId := 0;
  m_dwPRTSimulationThreadId := 0;
  m_bSHPRTReadyToRender := False;

  m_bShowHelp := True;
  m_bStopSimulator := false;
  m_fPercentDone := 0.0;

  m_pSHPRTEffect := nil;
  m_pSimpleLightingEffect := nil;

  m_pSphere := nil;
  m_pMesh := nil;
  m_pSHPRTBuffer := nil;
  m_aClusterBases := nil;
  m_aClusteredPCA := nil;
  ZeroMemory(@m_pPCAWeightTexture, SizeOf(m_pPCAWeightTexture));

  m_vDefaultLightDir[0] := D3DXVector3(0.37,0.36,-0.86);
  m_vDefaultLightDir[1] := D3DXVector3(-0.69,-0.60,-0.42);
  m_vDefaultLightDir[2] := D3DXVector3(-0.85,0.37,0.37);
  for i:= 0 to MAX_LIGHTS - 1 do
  begin
    m_vLightDir[i] := m_vDefaultLightDir[i];
    D3DXMatrixIdentity(m_mLightRot[i]);
    D3DXMatrixIdentity(m_mLightRotSnapshot[i]);
  end;
  m_nActiveLight := 0;
  m_nNumActiveLights := 1;
  m_fLightScale := 1.0;

  // Read persistent state information from registry
  // and if regkey is missing then set to default values
  RegCreateKeyEx(HKEY_CURRENT_USER, SAMPLE_REGKEY, 0, nil, REG_OPTION_NON_VOLATILE, KEY_READ or KEY_WRITE, nil, hRegKey, nil);
  DXUtil_GetDXSDKMediaPathCch(strMedia, MAX_PATH);
  DXUtil_ReadStringRegKeyCch(hRegKey, 'InitialDir', m_strInitialDir, MAX_PATH, strMedia);
  DXUtil_ReadStringRegKeyCch(hRegKey, 'InputMesh', m_strInputMesh, MAX_PATH, 'shapes1.x');
  DXUtil_ReadStringRegKeyCch(hRegKey, 'ResultsFile', m_strResultsFile, MAX_PATH, 'shapes1_results.dat');
  DXUtil_ReadIntRegKey(hRegKey, 'Order', m_dwOrder, 6);
  DXUtil_ReadIntRegKey(hRegKey, 'TextureSize', m_dwTextureSize, 256);
  DXUtil_ReadIntRegKey(hRegKey, 'TextureFormat', dwResult, DWORD(D3DFMT_Q16W16V16U16)); m_fmtTexture := TD3DFormat(dwResult);
  DXUtil_ReadIntRegKey(hRegKey, 'NumRays', m_dwNumRays, 256);
  DXUtil_ReadIntRegKey(hRegKey, 'NumRays', m_dwNumRays, 1024);
  DXUtil_ReadIntRegKey(hRegKey, 'NumBounces', m_dwNumBounces, 0);
  DXUtil_ReadIntRegKey(hRegKey, 'NumPCAVectors', m_dwNumPCAVectors, MAX_PCA_VECTORS);
  DXUtil_ReadIntRegKey(hRegKey, 'SubsurfaceScattering', dwResult, 0); m_bSubsurfaceScattering := (dwResult <> 0);
  DXUtil_ReadFloatRegKey(hRegKey, 'LengthScale', m_fLengthScale, 25.0);
  DXUtil_ReadIntRegKey(hRegKey, 'Spectral', dwResult, 1); m_bSpectral := (dwResult <> 0);
  DXUtil_ReadIntRegKey(hRegKey, 'PredefinedMatIndex', m_dwPredefinedMatIndex, 0);
  DXUtil_ReadFloatRegKey(hRegKey, 'RelativeIndexOfRefraction', m_fRelativeIndexOfRefraction, g_aPredefinedMaterials[0].fRelativeIndexOfRefraction);
  DXUtil_ReadFloatRegKey(hRegKey, 'DiffuseReflectanceR', m_DiffuseReflectance.r, g_aPredefinedMaterials[0].fDiffuseReflectance.r);
  DXUtil_ReadFloatRegKey(hRegKey, 'DiffuseReflectanceG', m_DiffuseReflectance.g, g_aPredefinedMaterials[0].fDiffuseReflectance.g);
  DXUtil_ReadFloatRegKey(hRegKey, 'DiffuseReflectanceB', m_DiffuseReflectance.b, g_aPredefinedMaterials[0].fDiffuseReflectance.b);
  DXUtil_ReadFloatRegKey(hRegKey, 'AbsoptionR', m_Absoption.r, g_aPredefinedMaterials[0].fAbsorptionCoefficient.r);
  DXUtil_ReadFloatRegKey(hRegKey, 'AbsoptionG', m_Absoption.g, g_aPredefinedMaterials[0].fAbsorptionCoefficient.g);
  DXUtil_ReadFloatRegKey(hRegKey, 'AbsoptionB', m_Absoption.b, g_aPredefinedMaterials[0].fAbsorptionCoefficient.b);
  DXUtil_ReadFloatRegKey(hRegKey, 'ReducedScatteringR', m_ReducedScattering.r, g_aPredefinedMaterials[0].fScatteringCoefficient.r);
  DXUtil_ReadFloatRegKey(hRegKey, 'ReducedScatteringG', m_ReducedScattering.g, g_aPredefinedMaterials[0].fScatteringCoefficient.g);
  DXUtil_ReadFloatRegKey(hRegKey, 'ReducedScatteringB', m_ReducedScattering.b, g_aPredefinedMaterials[0].fScatteringCoefficient.b);

  RegCloseKey(hRegKey);
end;

destructor CMyD3DApplication.Destroy;
var
  i: Integer;
begin
  for i:= 0 to MAX_LIGHTS-1 do
    FreeAndNil(m_LightArcBall[i]);
  FreeAndNil(m_Camera);
  inherited;
end;


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Paired with FinalCleanup().
//       The window has been created and the IDirect3D9 interface has been
//       created, but the device has not been created yet.  Here you can
//       perform application-related initialization and cleanup that does
//       not depend on a device.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit: HRESULT;
begin
  InitializeCriticalSection(m_cs);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Paired with DeleteDeviceObjects()
//       The device has been created.  Resources that are not lost on
//       Reset() can be created here -- resources in D3DPOOL_MANAGED,
//       D3DPOOL_SCRATCH, or D3DPOOL_SYSTEMMEM.  Image surfaces created via
//       CreateImageSurface are never lost and can be created here.  Vertex
//       shaders and pixel shaders can also be created here as they are not
//       lost on Reset().
//-----------------------------------------------------------------------------
function CMyD3DApplication.InitDeviceObjects: HRESULT;
var
  hDC: Windows.HDC;
  nHeight: Integer;
  dwShaderFlags: DWORD;
  str: array[0..MAX_PATH-1] of Char;
begin
  // Initialize the font
  hDC := GetDC(0);
  nHeight := -MulDiv(9, GetDeviceCaps(hDC, LOGPIXELSY), 72);
  ReleaseDC(0, hDC);
  Result := D3DXCreateFont(m_pd3dDevice, nHeight, 0, FW_BOLD, 0, False, DEFAULT_CHARSET,
                           OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH or FF_DONTCARE,
                           'Arial', m_pFont);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXCreateFont', Result, UnitName); Exit; end;

  // Create a sphere to show the light's position
  Result := D3DXCreateSphere(m_pd3dDevice, 1.0, 8, 8, m_pSphere, nil);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXCreateSphere', Result, UnitName); Exit; end;

  // Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders with the shader debugger.  
  // Debugging vertex shaders requires either REF or software vertex processing, and debugging 
  // pixel shaders requires REF.  The D3DXSHADER_FORCE_*_SOFTWARE_NOOPT flag improves the debug 
  // experience in the shader debugger.  It enables source level debugging, prevents instruction 
  // reordering, prevents dead code elimination, and forces the compiler to compile against the next 
  // higher available software target, which ensures that the unoptimized shaders do not exceed 
  // the shader model limitations.  Setting these flags will cause slower rendering since the shaders 
  // will be unoptimized and forced into software.  See the DirectX documentation for more information 
  // about using the shader debugger.
  dwShaderFlags := 0;
  {$IFDEF DEBUG_VS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_VS_SOFTWARE_NOOPT;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_PS_SOFTWARE_NOOPT;
  {$ENDIF}

  // Read the D3DX effect file
  DXUtil_FindMediaFileCb(str, SizeOf(str), 'SimpleLighting.fx');
  Result := D3DXCreateEffectFromFile(m_pd3dDevice, str, nil, nil,
                                     dwShaderFlags, nil, m_pSimpleLightingEffect, nil);

  // If this fails, there should be debug output as to 
  // they the .fx file failed to compile
  if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXCreateEffectFromFile', Result, UnitName); Exit; end;

  // This is code occurs when device is lost and the sample previously
  // viewing a mesh with PRT
  if m_bSHPRTReadyToRender then
  begin
    Result := LoadMeshAndResults;
    if FAILED(Result) then Exit;

    // If this fails, there should be debug output as to
    // they the .fx file failed to compile
    Result := LoadSHPRTEffect;
    if FAILED(Result) then begin Result:= DXTRACE_ERR('LoadSHPRTEffect', Result, UnitName); Exit; end;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DoStartupDlg()
// Desc: Allows the user to select to either run the simulator or 
//       load previously saved simulator results.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DoStartupDlg: HRESULT;
var
  nResult: Integer;
begin
  Pause(True);
  nResult := DialogBox(HInstance, MAKEINTRESOURCE(IDD_STARTUP), m_hWnd, @StaticStartupDlgProc);
  Pause(False);
  if (nResult = IDC_RUN_PRT_SIM) then
  begin
    // Ask the user about PRT simulator settings and launch
    // the PRT simulator in another thread to not block the UI thread
    Result := LaunchPRTSimulation;
    if FAILED(Result) then begin Result:= DXTRACE_ERR('LaunchPRTSimulation', Result, UnitName); Exit; end;
  end
  else if (nResult = IDC_VIEW_RESULTS) then 
  begin
    Result := LoadMeshAndResults;
    if FAILED(Result) then Exit;
  end;

  Result:= S_OK;
end;




//-----------------------------------------------------------------------------
// Name: DoCompressDlg()
// Desc: Allows the user to change the # of PCA vectors 
//-----------------------------------------------------------------------------
function CMyD3DApplication.DoCompressDlg: HRESULT;
var
  nResult: Integer;
begin
  if (not m_bWindowed) then
  begin
    // Go back to windowed mode to display dialog boxes
    Result := ToggleFullscreen;
    if FAILED(Result) then begin Result:= DXTRACE_ERR('ToggleFullscreen', Result, UnitName); Exit; end;
  end;

  Pause(true);
  nResult := DialogBox(HInstance, MAKEINTRESOURCE(IDD_COMPRESS), m_hWnd, @StaticCompressDlgProc);
  Pause(false);

  if (nResult = IDOK) and (m_hPRTSimulationThreadId = 0) then
  begin
    Result := CompressData;
    if FAILED(Result) then
      MessageBox(m_hWnd, 'Loading and compressing data failed'#10, 'Error', MB_OK);
  end;

  Result:= S_OK;
end;


//todo: Fill Bug request !!!
//-----------------------------------------------------------------------------
// Name: ViewResults()
// Desc: This function stops the simulator if its running, then loads the mesh,
//       loads the simulator results, compresses the simulator results.
//-----------------------------------------------------------------------------
function CMyD3DApplication.LoadMeshAndResults: HRESULT;
var
  nResult: Integer;
  dwResult: DWORD;
begin
  if (m_hPRTSimulationThreadId <> 0) then
  begin
    // Ask to stop the PRT simulator if it's running in the other thread
    dwResult := WaitForSingleObject(m_hPRTSimulationThreadId, 0);
    if (dwResult = WAIT_TIMEOUT) then
    begin
      nResult := MessageBox(m_hWnd, 'Simulation still processing.  Abort current simulation?', 'Warning', MB_YESNO or MB_ICONWARNING);
      if (nResult = IDNO) then
      begin
        Result:= S_OK;
        Exit;
      end;

      Result := StopPRTSimulation;
      if FAILED(Result) then Exit;
    end;
  end;

  SAFE_RELEASE(m_pMesh);
  SAFE_RELEASE(m_pSHPRTBuffer);

  // Load the mesh, and ensure it has a known decl
  Result := LoadMesh(m_strInputMesh, m_pMesh);
  if FAILED(Result) then
  begin
    MessageBox(m_hWnd, 'Couldn''t load mesh', 'Error', MB_OK );
    Result:= DXTRACE_ERR('LoadMesh', Result, UnitName);
    Exit;
  end;

  m_bSHPRTReadyToRender := false;

  // Load the previously saved SH PRT buffer from a file
  Result := ReadSHPRTBufferFromFile(m_strResultsFile, m_pSHPRTBuffer);
  if FAILED(Result) then
  begin
    MessageBox(m_hWnd, 'Couldn''t load simulator results file.  Run the simulator first to precompute the transfer vectors for this mesh.', 'Error', MB_OK);
    Result:= DXTRACE_ERR('ReadSHPRTBufferFromFile', Result, UnitName);
    Exit;
  end;

  // Compress the saved PRT simulator results
  Result := CompressData;
  if FAILED(Result) then begin Result:= DXTRACE_ERR('CompressData', Result, UnitName); Exit; end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: CompressData()
// Desc: This sets up the PCA data needed by the pixel shader
//-----------------------------------------------------------------------------
function CMyD3DApplication.CompressData: HRESULT;
var
  desc: TD3DXSHPRTBufferDesc;
  pPCABuffer: ID3DXBuffer;
  nChannelSize: Integer;
  nPCAVectorSize: Integer;
  nBasisSize: Integer;
  nBufferSize: Integer;
begin
  Result:= E_FAIL;

  // Ensure correct state
  if (m_pSHPRTBuffer = nil) or
     (m_pMesh = nil) or
     (m_hPRTSimulationThreadId <> 0) then Exit;

  // Get the description of the SH PRT buffer
  ZeroMemory(@desc, SizeOf(desc));
  Result := D3DXSHPRTExtractDesc(m_pSHPRTBuffer, desc);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXSHPRTExtractDesc', Result, UnitName); Exit; end;

  // Make note of the order used & if its spectral data
  m_dwOrder := desc.Order;
  m_bSpectral := (desc.NumChannels = 3);
  Assert(desc.Height = desc.Width); // sample requires this
  m_dwTextureSize := desc.Height;

  // Compress the results buffer using 1 cluster, and dwNumPCAVectors PCA vectors.
  // pPCABuffer will contain the compressed buffer.
  Result := D3DXSHPRTCompress(desc.Order, m_pSHPRTBuffer,
                              D3DXSHCQUAL_FASTLOWQUALITY, 1, m_dwNumPCAVectors,
                              pPCABuffer);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXSHPRTCompress', Result, UnitName); Exit; end;

  // Normalize the the compressed data.  This adjusts the mean vectors so
  // that the weights are between 0-1
  Result := D3DXSHPRTCompNormalizeData(pPCABuffer);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXSHPRTCompNormalizeData', Result, UnitName); Exit; end;

  // Setup the textures with PCA data
  Result := SetupTexWithPCAData(m_pMesh, pPCABuffer, desc, m_dwNumPCAVectors);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('SetupVBWithPCAData', Result, UnitName); Exit; end; //todo: Bug Report "SetupVBWithPCAData"

  // Extract the cluster bases into a large array of floats.
  // D3DXSHPRTCompExtractBasis will extract the basis
  // for a single cluster.  The basis for a cluster is an array of
  // (NumPCAVectors+1)*(NumChannels*Order^2) floats.
  // The "1+" is for the cluster mean.
  nChannelSize   := desc.Order * desc.Order;
  nPCAVectorSize := nChannelSize   * Integer(desc.NumChannels);
  nBasisSize     := nPCAVectorSize * Integer(m_dwNumPCAVectors+1);
  nBufferSize    := nBasisSize     * 1;

  FreeMem(m_aClusterBases);
  try
    GetMem(m_aClusterBases, SizeOf(Single)*nBufferSize);
  except
    Result:= DXTRACE_ERR('GetMem', E_OUTOFMEMORY, UnitName);
    Exit;
  end;

  // extract the basis of the cluster
  Result := D3DXSHPRTCompExtractBasis(0, pPCABuffer, PSingle(@m_aClusterBases[0]));
  if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXSHPRTCompExtractBasis', Result, UnitName); Exit; end;

  FreeMem(m_aClusteredPCA);
  try
    GetMem(m_aClusteredPCA, SizeOf(Single)*(1*(4+MAX_NUM_CHANNELS*m_dwNumPCAVectors)));
  except
    Result:= DXTRACE_ERR('GetMem', E_OUTOFMEMORY, UnitName);
    Exit;
  end;

  // Cleanup and allow rendering using the SH PRT data
  SAFE_RELEASE(pPCABuffer);

  m_bSHPRTReadyToRender := True;

  // If this fails, there should be debug output as to
  // they the .fx file failed to compile
  Result := LoadSHPRTEffect;
  if FAILED(Result) then begin Result:= DXTRACE_ERR('LoadSHPRTEffect', Result, UnitName); Exit; end;

  // Evaluate the lights in terms of SH coefficients 
  // and set the shader constants, and call this function
  // whenever the lights or object rotates and need to be reevaluated
  UpdateLightDir;
  Result := EvalLightsAndSetConstants;
  if FAILED(Result) then Exit;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: LoadMesh()
// Desc: Load the mesh and ensure the mesh has normals, and optimize
//       the mesh for this graphics card's vertex cache so the triangle list
//       inside in the mesh will cache miss less which improves perf.
//-----------------------------------------------------------------------------
function CMyD3DApplication.LoadMesh(strFileName: PChar; out ppMesh: ID3DXMesh): HRESULT;
var
  pMesh: ID3DXMesh;
  str: array[0..MAX_PATH-1] of Char;
  rgdwAdjacency: PDWORD;
  pTempMesh: ID3DXMesh;
  pVB: IDirect3DVertexBuffer9;
  pVertices: Pointer;
begin
  if (@ppMesh = nil) then
  begin
    Result:= E_INVALIDARG;
    Exit;
  end;

  // Load the mesh with D3DX and get back a ID3DXMesh*.  For this
  // sample we'll ignore the X file's embedded materials since we know
  // exactly the model we're loading.  See the mesh samples such as
  // "OptimizedMesh" for a more generic mesh loading example.
  Result := DXUtil_FindMediaFileCb(str, SizeOf(str), strFileName);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('DXUtil_FindMediaFileCb', D3DAPPERR_MEDIANOTFOUND, UnitName); Exit; end;
  Result := D3DXLoadMeshFromX(str, D3DXMESH_MANAGED,
                              m_pd3dDevice, nil, nil, nil, nil, pMesh);
  if FAILED(Result) or (pMesh = nil) then begin Result:= DXTRACE_ERR('D3DXLoadMeshFromX', Result, UnitName); Exit; end;

  // Make sure there are texture coordinates which are required for texturing
  if (((pMesh.GetFVF and D3DFVF_TEXCOUNT_MASK) shr D3DFVF_TEXCOUNT_SHIFT) = 0) then
  begin
    SAFE_RELEASE(m_pMesh);
    Result:= E_FAIL;
    Exit;
  end;

  // Lock the vertex buffer to get the object's radius & center
  // simply to help position the camera a good distance away from the mesh.
  Result := pMesh.GetVertexBuffer(pVB);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('GetVertexBuffer', Result, UnitName); Exit; end;
  Result := pVB.Lock(0, 0, pVertices, 0);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('Lock', Result, UnitName); Exit; end;
  Result := D3DXComputeBoundingSphere(PD3DXVector3(pVertices), pMesh.GetNumVertices,
                                  D3DXGetFVFVertexSize(pMesh.GetFVF), m_vObjectCenter,
                                  m_fObjectRadius);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXComputeBoundingSphere', Result, UnitName); Exit; end;
  pVB.Unlock;
  SAFE_RELEASE(pVB);

  // Update camera's viewing radius based on the object radius
  m_Camera.SetRadius(m_fObjectRadius*3.0, m_fObjectRadius*1.1, m_fObjectRadius*20.0);
  m_Camera.SetModelCenter(m_vObjectCenter);

  // Make sure there are normals which are required for lighting
  if (pMesh.GetFVF and D3DFVF_NORMAL = 0) then
  begin
    Result := pMesh.CloneMeshFVF(pMesh.GetOptions,
                                 pMesh.GetFVF or D3DFVF_NORMAL,
                                 m_pd3dDevice, pTempMesh);
    if FAILED(Result) then begin Result:= DXTRACE_ERR('CloneMeshFVF', Result, UnitName); Exit; end;

    Result := D3DXComputeNormals(pTempMesh, nil);
    //todo: In Original C++ code there is bug in this place: "CloneMeshFVF" mentioned
    if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXComputeNormals', Result, UnitName); Exit; end;

    SAFE_RELEASE(pMesh);
    pMesh := pTempMesh;
  end;

  // Optimize the mesh for this graphics card's vertex cache
  // so when rendering the mesh's triangle list the vertices will
  // cache hit more often so it won't have to re-execute the vertex shader
  // on those vertices so it will improves perf.
  try
    GetMem(rgdwAdjacency, SizeOf(DWORD) * pMesh.GetNumFaces * 3);
  except
    Result:= E_OUTOFMEMORY;
    Exit;
  end;
  pMesh.ConvertPointRepsToAdjacency(nil, rgdwAdjacency);
  Result:= pMesh.OptimizeInplace(D3DXMESHOPT_VERTEXCACHE, rgdwAdjacency, nil, nil, nil);
  FreeMem(rgdwAdjacency);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('OptimizeInplace', Result, UnitName);
    Exit;
  end;

  ppMesh := pMesh;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: SetupTexWithPCAData()
// Desc: Create and fill textures with PCA data
//-----------------------------------------------------------------------------
function CMyD3DApplication.SetupTexWithPCAData(pMesh: ID3DXMesh; pPCABuffer: ID3DXBuffer; desc: TD3DXSHPRTBufferDesc; dwNumPCAVectors: DWORD): HRESULT;
var
 i: DWORD;
begin
  // Each texture can how four PCA weights
  for i:= 0 to m_dwNumPCAVectors div 4 - 1 do
  begin
    SAFE_RELEASE(m_pPCAWeightTexture[i]);

    // Create the texture with complete mipmapping
    Result := D3DXCreateTexture(m_pd3dDevice, m_dwTextureSize, m_dwTextureSize, D3DX_DEFAULT, 0,
                                m_fmtTexture, D3DPOOL_MANAGED, m_pPCAWeightTexture[i]);
    if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXCreateTexture', Result, UnitName); Exit; end;

    // Fill the texture with 4 PCA weights starting at PCA weight i*4.
    Result := D3DXSHPRTCompExtractTexture(i*4, 4, pPCABuffer, m_pPCAWeightTexture[i]);
    if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXSHPRTCompExtractTexture', Result, UnitName); Exit; end;

    // Filter the PCA weights to rest of the mipmap levels
    Result := D3DXFilterTexture(m_pPCAWeightTexture[i], nil, D3DX_DEFAULT, D3DX_DEFAULT);
    if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXFilterTexture', Result, UnitName); Exit; end;

    // Set the sampler state
    m_pd3dDevice.SetTexture(i, m_pPCAWeightTexture[i]);
    m_pd3dDevice.SetSamplerState(i, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
    m_pd3dDevice.SetSamplerState(i, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
    m_pd3dDevice.SetSamplerState(i, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
    m_pd3dDevice.SetSamplerState(i, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
    m_pd3dDevice.SetSamplerState(i, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
  end;

  Result:= S_OK;
end;



//-----------------------------------------------------------------------------
// Name: LoadSHPRTEffect()
// Desc: Load the SHPRTPixel.fx effect file
//-----------------------------------------------------------------------------
function CMyD3DApplication.LoadSHPRTEffect: HRESULT;
var
  dwNumPConsts: DWORD;
  aDefines: array[0..1] of TD3DXMacro;
  szMaxNumPCAVectors: array[0..63] of Char;
  dwShaderFlags: DWORD;
  str: array[0..MAX_PATH-1] of Char;
begin
  // The number of pixel shader consts need by the shader can't exceed the 
  // amount the HW can support
  dwNumPConsts := 1 * (1 + MAX_NUM_CHANNELS*m_dwNumPCAVectors div 4) + 4;
  if (dwNumPConsts > 32) then
  begin
    Result:= E_FAIL;
    Exit;
  end;

  SAFE_RELEASE(m_pSHPRTEffect);

  StrLFmt(szMaxNumPCAVectors, 64, '%d', [m_dwNumPCAVectors]);
  szMaxNumPCAVectors[63] := #0;
  aDefines[0].Name       := 'NUM_PCA_VECTORS';
  aDefines[0].Definition := szMaxNumPCAVectors;
  aDefines[1].Name       := nil;
  aDefines[1].Definition := nil;

  // Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders with the shader debugger.
  // Debugging vertex shaders requires either REF or software vertex processing, and debugging
  // pixel shaders requires REF.  The D3DXSHADER_FORCE_*_SOFTWARE_NOOPT flag improves the debug
  // experience in the shader debugger.  It enables source level debugging, prevents instruction
  // reordering, prevents dead code elimination, and forces the compiler to compile against the next
  // higher available software target, which ensures that the unoptimized shaders do not exceed
  // the shader model limitations.  Setting these flags will cause slower rendering since the shaders
  // will be unoptimized and forced into software.  See the DirectX documentation for more information
  // about using the shader debugger.
  dwShaderFlags := 0;
  {$IFDEF DEBUG_VS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_VS_SOFTWARE_NOOPT;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_PS_SOFTWARE_NOOPT;
  {$ENDIF}

  // Read the D3DX effect file
  DXUtil_FindMediaFileCb(str, SizeOf(str), 'SHPRTPixel.fx');
  Result := D3DXCreateEffectFromFile(m_pd3dDevice, str, @aDefines, nil,
                                     dwShaderFlags, nil, m_pSHPRTEffect, nil);

  // If this fails, there should be debug output as to
  // they the .fx file failed to compile
  if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXCreateEffectFromFile', Result, UnitName); Exit; end;

  // Make sure the technique works on this card
  Result := m_pSHPRTEffect.ValidateTechnique('PrecomputedSHLighting');
  if FAILED(Result) then begin Result:= DXTRACE_ERR('ValidateTechnique', Result, UnitName); Exit; end;

  Result:= S_OK;
end;




//-----------------------------------------------------------------------------
// Name: EvalLightsAndSetConstants()
// Desc: Evaluate the lights in terms of SH coefficients
//       and set the shader constants, and call this function
//       whenever the lights or object rotates and need to be reevaluated
//-----------------------------------------------------------------------------
function CMyD3DApplication.EvalLightsAndSetConstants: HRESULT;
var
  lightColor: TD3DXColor;
  fLight: array[0..MAX_LIGHTS-1, 0..2, 0..MAX_SH_ORDER*MAX_SH_ORDER-1] of Single;  
  lightDirObjectSpace: TD3DXVector3;
  mWorldInv: TD3DXMatrix;
  i: Integer;
begin
  if not m_bSHPRTReadyToRender or
     (m_aClusterBases = nil) or
     (m_pMesh = nil) or
     (m_pSHPRTEffect = nil) then
  begin
    Result:= S_OK;
    Exit;
  end;

  lightColor:= D3DXColor(1.0, 1.0, 1.0, 1.0);
  D3DXColorScale(lightColor, lightColor, m_fLightScale);

  // Pass in the light direction, the intensity of each channel, and it returns
  // the source radiance as an array of order^2 SH coefficients for each color channel.
  D3DXMatrixInverse(mWorldInv, nil, m_mWorld);

  for i:= 0 to m_nNumActiveLights - 1 do
  begin
    // Transform the world space light dir into object space
    // Note that if there's multiple objects using PRT in the scene, then
    // for each object you need to either evaulate the lights in object space
    // evaulate the lights in world and rotate the light coefficients
    // into object space.
    D3DXVec3TransformNormal(lightDirObjectSpace, m_vLightDir[i], mWorldInv);

    // This sample uses D3DXSHEvalDirectionalLight(), but there's other
    // types of lights provided by D3DXSHEval*.  Pass in the
    // order of SH, color of the light, and the direction of the light
    // in object space.
    // The output is the source radiance coefficients for the SH basis functions.
    // There are 3 outputs, one for each channel (R,G,B).
    // Each output is an array of m_dwOrder^2 floats.
    Result := D3DXSHEvalDirectionalLight(m_dwOrder, lightDirObjectSpace,
                                         lightColor.r, lightColor.g, lightColor.b,
                                         @fLight[i][0], @fLight[i][1], @fLight[i][2]);
    if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXSHEvalDirectionalLight', Result, UnitName); Exit; end;
  end;

  // For multiple lights, just them sum up using D3DXSHAdd().
  for i:= 1 to m_nNumActiveLights - 1 do
  begin
    // D3DXSHAdd will add Order^2 floats.  There are 3 color channels,
    // so call it 3 times.
    D3DXSHAdd(@fLight[0][0], m_dwOrder, @fLight[0][0], @fLight[i][0]);
    D3DXSHAdd(@fLight[0][1], m_dwOrder, @fLight[0][1], @fLight[i][1]);
    D3DXSHAdd(@fLight[0][2], m_dwOrder, @fLight[0][2], @fLight[i][2]);
  end;

  // Setup the shader constants based on the source radiance coefficients and
  // the cluster basis.
  Result := SetShaderConstants(@fLight[0][0], @fLight[0][1], @fLight[0][2]);
  if FAILED(Result) then Exit;

  Result:= S_OK;
end;




//-----------------------------------------------------------------------------
// Name: SetShaderConstants()
// Desc: Setup the shader constants based on the source radiance
//       coefficients and the cluster basis.
//-----------------------------------------------------------------------------
function CMyD3DApplication.SetShaderConstants(pfRLC, pfGLC, pfBLC: PSingle): HRESULT;
var
  dwNumChannels: DWORD;
  nChannelSize: Integer;
  nPCAVectorSize: Integer;
  nBufferSize: Integer;
  nChannelMult: Integer;
  pPCAStart: PSingleArray;
  iPCA: DWORD;
  nOffset: Integer;
begin
  if m_bSpectral then dwNumChannels := 3
    else dwNumChannels := 1;
  nChannelSize   := m_dwOrder * m_dwOrder;
  nPCAVectorSize := nChannelSize * Integer(dwNumChannels);
  nBufferSize    := 1 * (4+MAX_NUM_CHANNELS*m_dwNumPCAVectors);
  if m_bSpectral then nChannelMult := nChannelSize else nChannelMult := 0;

  // Compute an array of floats to pass as constants to the vertex shader.
  // This array is the source radiance dotted with the transfer function.
  // The source radiance is the lighting environment in terms of spherical
  // harmonic coefficients which comes from D3DXSHEvalDirectionalLight, and
  // the transfer function are in terms of spherical harmonic basis coefficients
  // that comes from the D3DXSHPRTCompExtractBasis().
  // First we compute L dot Mk, where Mk is the mean of the cluster
  m_aClusteredPCA[0] := D3DXSHDot(m_dwOrder, @m_aClusterBases[0*nChannelMult], pfRLC);
  m_aClusteredPCA[1] := D3DXSHDot(m_dwOrder, @m_aClusterBases[1*nChannelMult], pfGLC);
  m_aClusteredPCA[2] := D3DXSHDot(m_dwOrder, @m_aClusterBases[2*nChannelMult], pfBLC);
  m_aClusteredPCA[3] := 0.0;

  // Then we compute L dot Bkj, where Bkj is the
  // jth PCA basis vector for cluster k
  pPCAStart := @m_aClusteredPCA[4];
  for iPCA := 0 to m_dwNumPCAVectors - 1 do
  begin
    nOffset := Integer(iPCA+1)*nPCAVectorSize;

    pPCAStart[0*m_dwNumPCAVectors + iPCA] := D3DXSHDot(m_dwOrder, @m_aClusterBases[nOffset + 0*nChannelMult], pfRLC);
    pPCAStart[1*m_dwNumPCAVectors + iPCA] := D3DXSHDot(m_dwOrder, @m_aClusterBases[nOffset + 1*nChannelMult], pfGLC);
    pPCAStart[2*m_dwNumPCAVectors + iPCA] := D3DXSHDot(m_dwOrder, @m_aClusterBases[nOffset + 2*nChannelMult], pfBLC);
  end;

  Result := m_pSHPRTEffect.SetFloatArray('vClusteredPCA', @m_aClusteredPCA[0], nBufferSize);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('SetFloatArray', Result, UnitName); Exit; end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Paired with InvalidateDeviceObjects()
//       The device exists, but may have just been Reset().  Resources in
//       D3DPOOL_DEFAULT and any other device state that persists during
//       rendering should be set here.  Render states, matrices, textures,
//       etc., that don't change during rendering can be set once here to
//       avoid redundant state setting during Render() or FrameMove().
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects: HRESULT;
var
  i: Integer;
  fAspect: Single;
begin
  if Assigned(m_pFont) then
  begin
    Result:= m_pFont.OnResetDevice;
    if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pFont.OnResetDevice', Result, UnitName); Exit; end;
  end;

  if m_bWireFrame
    then m_pd3dDevice.SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME)
    else m_pd3dDevice.SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);

  for i:= 0 to MAX_LIGHTS - 1 do
    m_LightArcBall[i].SetWindow(m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height);

  // Setup the camera
  fAspect := m_d3dsdBackBuffer.Width / m_d3dsdBackBuffer.Height;
  m_Camera.SetProjParams(D3DX_PI/4, fAspect, 1.0, 10000.0);
  m_Camera.SetWindow(m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height);
  m_Camera.SetButtonMasks(MOUSE_LEFT_BUTTON, MOUSE_WHEEL, MOUSE_MIDDLE_BUTTON);
  m_Camera.SetEnablePositionMovement(False);
  m_Camera.SetResetCursorAfterMove(False);

  Result:= m_pSimpleLightingEffect.OnResetDevice;
  if FAILED(Result) then begin Result:= DXTRACE_ERR('OnResetDevice', Result, UnitName); Exit; end;

  // This is code occurs when device is reset and the sample previously
  // viewing a mesh with PRT
  if m_bSHPRTReadyToRender then
  begin
    Result := m_pSHPRTEffect.OnResetDevice;
    if FAILED(Result) then begin Result:= DXTRACE_ERR('OnResetDevice', Result, UnitName); Exit; end;

    // Set the texture/sampler states
    for i:= 0 to m_dwNumPCAVectors div 4 do 
    begin
      // Set the sampler state
      m_pd3dDevice.SetTexture(i, m_pPCAWeightTexture[i]);
      m_pd3dDevice.SetSamplerState(i, D3DSAMP_MIPFILTER, D3DTEXF_NONE);
      m_pd3dDevice.SetSamplerState(i, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
      m_pd3dDevice.SetSamplerState(i, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
      m_pd3dDevice.SetSamplerState(i, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
      m_pd3dDevice.SetSamplerState(i, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
    end;

    // Evaluate the lights in terms of SH coefficients
    // and set the shader constants, and call this function
    // whenever the lights or object rotates and need to be reevaluated
    Result := EvalLightsAndSetConstants;
    if FAILED(Result) then Exit;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove: HRESULT;
begin
  // Update the camera's postion based on user input
  m_Camera.FrameMove(m_fElapsedTime);

  // Get the world, view, projection matrix from the camera class
  m_mWorld := m_Camera.GetWorldMatrix^;
  m_mView  := m_Camera.GetViewMatrix^;
  m_mProj  := m_Camera.GetProjMatrix^;

  // Show some dialog boxes if the state is right
  Result:= UpdateUI;
  if FAILED(Result) then Exit;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: UpdateUI()
// Desc: Show some dialog boxes if the state is right
//-----------------------------------------------------------------------------
var
  s_bFirstTime: Boolean = True; // static bool s_bFirstTime = true;

function CMyD3DApplication.UpdateUI: HRESULT;
var
  dwResult: DWORD;
  hMenu: Windows.HMENU;
begin
  // If this is the first time call, then show the startup dialog
  if s_bFirstTime then
  begin
    DoStartupDlg;
    s_bFirstTime := False;
  end;

  // Check to see if simulation is still running
  if (m_hPRTSimulationThreadId <> 0) then
  begin
    dwResult := WaitForSingleObject(m_hPRTSimulationThreadId, 0);
    if (dwResult <> WAIT_TIMEOUT) then
    begin
      // Simulation completed
      hMenu := GetSubMenu(GetMenu(m_hWnd), 1);
      EnableMenuItem(hMenu, IDM_ABORT_SIMULATION, MF_GRAYED);
      DrawMenuBar(m_hWnd);

      m_nTechnique := 0;
      m_bPRTSimulationComplete := True;
      m_hPRTSimulationThreadId := 0;
      m_dwPRTSimulationThreadId := 0;

      if (not m_bPRTSimulationFailed) then
      begin
        Result := CompressData;
        if FAILED(Result) then begin Result:= DXTRACE_ERR('CompressData', Result, UnitName); Exit; end;
      end;
    end else
    begin
      // Yield time to the other thread
      Sleep(10);
    end;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
var
  mWorldViewProj: TD3DXMatrixA16;
  i: Integer;
  sphereColor: TD3DXColor;
  m: TD3DXMatrixA16;
begin
  // Clear the render target
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER, $00003F3F, 1.0, 0);

  // Begin the scene
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    // Render the light spheres so the user can
    // visually see the light dir
    for i:= 0 to m_nNumActiveLights - 1 do
    begin
      if (i = m_nActiveLight)
        then sphereColor := D3DXColor(1,1,0,1)
        else sphereColor := D3DXColor(1,1,1,1);
      RenderLightSphere(m_vLightDir[i], sphereColor);
    end;

    // mWorldViewProj = m_mWorld * m_mView * m_mProj;
    D3DXMatrixMultiply(m, m_mView, m_mProj);
    D3DXMatrixMultiply(mWorldViewProj, m_mWorld, m);

    m_pSimpleLightingEffect.SetMatrix('mWorldViewProjection', mWorldViewProj);
    if (m_pSHPRTEffect <> nil) then
      m_pSHPRTEffect.SetMatrix('mWorldViewProjection', mWorldViewProj);

    // Render the scene using either N.L lighting or PRT
    case m_nTechnique of
      0:
      begin
        Result:= RenderMeshWithSHPRT;
        if FAILED(Result) then Exit;
      end;
      1:
      begin
        Result:= RenderMeshWithSimpleLighting;
        if FAILED(Result) then Exit;
      end;
    end;

    // Render the text
    RenderText;

    Result := m_pd3dDevice.EndScene;
    if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pd3dDevice.EndScene', Result, UnitName); Exit; end;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RenderLightSphere()
// Desc: Render the light spheres so the user can visually see the light dir
//-----------------------------------------------------------------------------
function CMyD3DApplication.RenderLightSphere(lightDir: TD3DXVector3; sphereColor: TD3DXColor): HRESULT;
var
  iPass, cPasses: LongWord;
  mScale: TD3DXMatrix;
  mTrans: TD3DXMatrix;
  mWorldViewProj: TD3DXMatrixA16;
  lightOff: TD3DXColor;
  vL: TD3DXVector3;
  m: TD3DXMatrix;
  mtrlAmbientColor: TD3DXColor;
begin
  m_pSimpleLightingEffect.SetTechnique('SimpleLighting');

  lightOff:= D3DXColor(0,0,0,0);
  m_pSimpleLightingEffect.SetVector('Light1DiffuseColor',   PD3DXVector4(@lightOff)^);
  m_pSimpleLightingEffect.SetVector('Light2DiffuseColor',   PD3DXVector4(@lightOff)^);
  m_pSimpleLightingEffect.SetVector('Light3DiffuseColor',   PD3DXVector4(@lightOff)^);
  m_pSimpleLightingEffect.SetVector('MaterialAmbientColor', PD3DXVector4(@sphereColor)^);

  D3DXVec3Scale(vL, lightDir, m_fObjectRadius * 1.2);
  D3DXMatrixTranslation(mTrans, vL.x, vL.y, vL.z);
  D3DXMatrixScaling(mScale, m_fObjectRadius*0.05, m_fObjectRadius*0.05, m_fObjectRadius*0.05);

  // mWorldViewProj = mScale * mTrans * m_mView * m_mProj;
  D3DXMatrixMultiply(m, m_mView, m_mProj);
  D3DXMatrixMultiply(m, mTrans, m);
  D3DXMatrixMultiply(mWorldViewProj, mScale, m);
  m_pSimpleLightingEffect.SetMatrix('mWorldViewProjection', mWorldViewProj);

  Result := m_pSimpleLightingEffect._Begin(@cPasses, 0);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pSimpleLightingEffect._Begin', Result, UnitName); Exit; end;
  for iPass := 0 to cPasses - 1 do
  begin
    Result := m_pSimpleLightingEffect.Pass(iPass);
    if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pSimpleLightingEffect.Pass', Result, UnitName); Exit; end;
    Result := m_pSphere.DrawSubset(0);
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawSubset', Result, UnitName); Exit; end;
  end;
  m_pSimpleLightingEffect._End;

  mtrlAmbientColor := D3DXColor(0,0,0,0);
  m_pSimpleLightingEffect.SetVector('MaterialAmbientColor', PD3DXVector4(@mtrlAmbientColor)^);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RenderMeshWithSHPRT()
// Desc: Render the scene with per pixel PRT 
//-----------------------------------------------------------------------------
function CMyD3DApplication.RenderMeshWithSHPRT: HRESULT;
var
  iPass, cPasses, i: LongWord;
  dwAttribs: DWORD;
begin
  if not m_bSHPRTReadyToRender or (m_aClusterBases = nil) or (m_pMesh = nil) or (m_pSHPRTEffect = nil) then
  begin
    Result:= S_OK;
    Exit;
  end;

  // Set the technique and render the scene
  m_pSHPRTEffect.SetTechnique('PrecomputedSHLighting');

  Result := m_pSHPRTEffect._Begin(@cPasses, 0);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pSHPRTEffect._Begin', Result, UnitName); Exit; end;
  for iPass := 0 to cPasses - 1 do
  begin
    Result := m_pSHPRTEffect.Pass(iPass);
    if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pSHPRTEffect.Pass', Result, UnitName); Exit; end;

    dwAttribs := 0;
    m_pMesh.GetAttributeTable( nil, @dwAttribs);
    for i:= 0 to dwAttribs - 1 do
    begin
      Result := m_pMesh.DrawSubset(0);
      if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pMesh.DrawSubset', Result, UnitName); Exit; end;
    end;
  end;

  m_pSHPRTEffect._End;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RenderMeshWithSimpleLighting()
// Desc: Render the scene with standard diffuse (N.L) lighting
//-----------------------------------------------------------------------------
function CMyD3DApplication.RenderMeshWithSimpleLighting: HRESULT;
var
  iPass, cPasses: LongWord;
  lightDirObjectSpace: TD3DXVector3;
  lightOn: TD3DXColor;
  lightOff: TD3DXColor;
  mWorldInv: TD3DXMatrix;
  dwAttribs: DWORD;
  i: Integer;
begin
  lightOn:= D3DXColor(1,1,1,1);
  lightOff:= D3DXColor(0,0,0,0);

  D3DXColorScale(lightOn, lightOn, m_fLightScale);

  if (m_pMesh = nil) then
  begin
    Result:= S_OK;
    Exit;
  end;

  m_pSimpleLightingEffect.SetTechnique('SimpleLighting');

  D3DXMatrixInverse(mWorldInv, nil, m_mWorld);

  // Light 1
  D3DXVec3TransformNormal(lightDirObjectSpace, m_vLightDir[0], mWorldInv);
  m_pSimpleLightingEffect.SetFloatArray('Light1Dir', PSingle(@lightDirObjectSpace.x), 3);
  m_pSimpleLightingEffect.SetVector('Light1DiffuseColor', PD3DXVector4(@lightOn)^);

  // Light 2
  D3DXVec3TransformNormal(lightDirObjectSpace, m_vLightDir[1], mWorldInv);
  m_pSimpleLightingEffect.SetFloatArray('Light2Dir', PSingle(@lightDirObjectSpace.x), 3);
  if (m_nNumActiveLights > 1)
    then m_pSimpleLightingEffect.SetVector('Light2DiffuseColor', PD3DXVector4(@lightOn)^)
    else m_pSimpleLightingEffect.SetVector('Light2DiffuseColor', PD3DXVector4(@lightOff)^);

  // Light 3
  D3DXVec3TransformNormal(lightDirObjectSpace, m_vLightDir[2], mWorldInv);
  m_pSimpleLightingEffect.SetFloatArray('Light3Dir', PSingle(@lightDirObjectSpace.x), 3);
  if (m_nNumActiveLights > 2)
    then m_pSimpleLightingEffect.SetVector('Light3DiffuseColor', PD3DXVector4(@lightOn)^)
    else m_pSimpleLightingEffect.SetVector('Light3DiffuseColor', PD3DXVector4(@lightOff)^);

  // Render the scene using simple N dot L lighting (up to 3 lights)
  Result := m_pSimpleLightingEffect._Begin(@cPasses, 0);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pSimpleLightingEffect._Begin', Result, UnitName); Exit; end;
  for iPass := 0 to cPasses - 1 do
  begin
    Result := m_pSimpleLightingEffect.Pass(iPass);
    if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pSimpleLightingEffect.Pass', Result, UnitName); Exit; end;

    dwAttribs := 0;
    m_pMesh.GetAttributeTable(nil, @dwAttribs);
    for i:= 0 to dwAttribs - 1 do
    begin
      Result := m_pMesh.DrawSubset(i);
      if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pMesh.DrawSubset', Result, UnitName); Exit; end;
    end;
  end;
  m_pSimpleLightingEffect._End;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RenderText()
// Desc: Draw the help & statistics for running sample
//-----------------------------------------------------------------------------
function CMyD3DApplication.RenderText: HRESULT;
var
  rc: TRect;
  iLineY: Integer;
  sz: array[0..255] of Char;
  dwNumPConsts: DWORD;
  fPercentDone: Single;
begin
  if (nil = m_pFont) then
  begin
    Result:= S_FALSE;
    Exit;
  end;

{$IFDEF UNICODE}
  For UNICODE you should use "m_pFont.DrawTextW" routine!!!
{$ENDIF}

  // Output statistics
  iLineY := 0;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, m_strFrameStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, m_strDeviceStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

  case m_nTechnique of
    0: StrCopy(sz, 'Technique: RenderMeshWithSHPRT');
    1: StrCopy(sz, 'Technique: RenderMeshWithSimpleLighting');
  end;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, sz, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

  StrLFmt(sz, 256, 'Number of lights: %d, Light scale: %0.2f, Light dir: <%0.2f,%0.2f,%0.2f>',
    [m_nNumActiveLights, m_fLightScale, m_vLightDir[m_nActiveLight].x, m_vLightDir[m_nActiveLight].y, m_vLightDir[m_nActiveLight].z]);
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, sz, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

  dwNumPConsts := 1 * (1 + MAX_NUM_CHANNELS*m_dwNumPCAVectors div 4) + 4;
  StrLFmt(sz, 256, '# PCA vectors: %d, # Pixel Shader Constants (32 max): %d', [m_dwNumPCAVectors, dwNumPConsts]);
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, sz, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

  StrLFmt(sz, 256, 'Texture Size: %d, Texture Format: %s',
    [m_dwTextureSize, D3DUtil_D3DFormatToString(m_fmtTexture, False)]);
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, sz, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

  if (m_hPRTSimulationThreadId <> 0) then
  begin
    // Keep 1 thread accessing the shared data at a time
    EnterCriticalSection(m_cs);
    fPercentDone := m_fPercentDone;
    LeaveCriticalSection(m_cs);

    StrLFmt(sz, 256, 'PRT simulation in progress. %0.1f%% done.  It may take a while...'#10, [fPercentDone*100.0]);
    SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
    Result:= m_pFont.DrawTextA(nil, sz, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
      if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
  end
  else if (m_bPRTSimulationComplete and not m_bPRTSimulationFailed) then
  begin
    StrCopy(sz, 'PRT simulation complete!');
    SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
    Result:= m_pFont.DrawTextA(nil, sz, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
      if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
  end;

  if (m_pMesh = nil) or (not m_bSHPRTReadyToRender and (m_hPRTSimulationThreadId = 0)) then
  begin
    StrCopy(sz, 'Use the PRT menu to load a mesh with simulator results');
  end else
  begin
    case m_nTechnique of
      0:
      begin
        if (m_bSHPRTReadyToRender)
        then StrCopy(sz, 'This is per pixel PRT.  Press ''N'' to change to simple N.L lighting')
        else StrCopy(sz, 'Simulation in progress.  Press ''N'' to change to simple lighting');
      end;
      1:
      begin
        if m_bSHPRTReadyToRender
        then StrCopy(sz, 'This is simple N.L lighting.  Press ''N'' to change to per pixel PRT')
        else StrCopy(sz, 'This is simple N.L lighting.  SH PRT simulation in progress');
      end;
    end;
  end;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, sz, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

  // Draw help
  if m_bShowHelp then
  begin
    iLineY := m_d3dsdBackBuffer.Height-15*6;
    SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
    Result:= m_pFont.DrawTextA(nil, 'Controls:', -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));
      if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
    SetRect(rc, 20, iLineY, 0, 0);
    Result:= m_pFont.DrawTextA(nil, 'Rotate model: Left mouse button'#10 +
                                    'Rotate camera: Right mouse button'#10 +
                                    'Rotate active light: Right mouse button'#10 +
                                    'Zoom camera: Mouse wheel scroll'#10 +
                                    'Change # of active lights: L'#10,
                               -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

    SetRect(rc, 250, iLineY, 0, 0);
    Result:= m_pFont.DrawTextA(nil, 'Change # of active lights: L'#10 +
                                    'Change light scale: U,I,O'#10 +
                                    'Compression settings: C,E,R'#10 +
                                    'Change render technique: N'#10 +
                                    'Wireframe toggle: W'#10,
                               -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

    SetRect(rc, 450, iLineY, 0, 0);
    Result:= m_pFont.DrawTextA(nil, 'Hide help: F1'#10 +
                                    'Change Device: F2'#10 +
                                    'View readme: F9'#10 +
                                    'Quit: ESC',
                               -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
  end else
  begin
    SetRect(rc, 2, iLineY, 0, 0); // Inc(iLineY, 15); //BAA: never used anyway
    Result:= m_pFont.DrawTextA(nil, 'Press F1 for help',
                               -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Invalidates device objects.  Paired with RestoreDeviceObjects()
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects: HRESULT;
begin
  if Assigned(m_pFont) then m_pFont.OnLostDevice;
  if Assigned(m_pSimpleLightingEffect) then m_pSimpleLightingEffect.OnLostDevice;
  if Assigned(m_pSHPRTEffect) then m_pSHPRTEffect.OnLostDevice;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Paired with InitDeviceObjects()
//       Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects: HRESULT;
var
  i: Integer;
begin
  SAFE_RELEASE(m_pSphere);
  SAFE_RELEASE(m_pFont);
  SAFE_RELEASE(m_pMesh);
  FreeMem(m_aClusterBases);
  for i:= 0 to MAX_PCA_VECTORS div 4 - 1 do
    SAFE_RELEASE(m_pPCAWeightTexture[i]);

  SAFE_RELEASE(m_pSimpleLightingEffect);
  SAFE_RELEASE(m_pSHPRTEffect);

  // Stop the PRT simulator if it's running in the other thread
  StopPRTSimulation;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Paired with OneTimeSceneInit()
//       Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
var
  hRegKey: HKEY;
begin
  DeleteCriticalSection(m_cs);
  SAFE_RELEASE(m_pSHPRTBuffer);

  // Write persistent state information from registry
  RegCreateKeyEx(HKEY_CURRENT_USER, SAMPLE_REGKEY, 0, nil, REG_OPTION_NON_VOLATILE, KEY_READ or KEY_WRITE, nil, hRegKey, nil);
  DXUtil_WriteIntRegKey(hRegKey, 'Order', m_dwOrder);
  DXUtil_WriteIntRegKey(hRegKey, 'TextureSize', m_dwTextureSize);
  DXUtil_WriteIntRegKey(hRegKey, 'TextureFormat', DWORD(m_fmtTexture));
  DXUtil_WriteIntRegKey(hRegKey, 'NumRays', m_dwNumRays);
  DXUtil_WriteIntRegKey(hRegKey, 'NumBounces', m_dwNumBounces);
  DXUtil_WriteIntRegKey(hRegKey, 'NumPCAVectors', m_dwNumPCAVectors);
  DXUtil_WriteIntRegKey(hRegKey, 'SubsurfaceScattering', Ord(m_bSubsurfaceScattering));
  DXUtil_WriteFloatRegKey(hRegKey, 'LengthScale', m_fLengthScale);
  DXUtil_WriteIntRegKey(hRegKey, 'Spectral', Ord(m_bSpectral));
  DXUtil_WriteIntRegKey(hRegKey, 'PredefinedMatIndex', m_dwPredefinedMatIndex);
  DXUtil_WriteFloatRegKey(hRegKey, 'DiffuseReflectanceR', m_DiffuseReflectance.r);
  DXUtil_WriteFloatRegKey(hRegKey, 'DiffuseReflectanceG', m_DiffuseReflectance.g);
  DXUtil_WriteFloatRegKey(hRegKey, 'DiffuseReflectanceB', m_DiffuseReflectance.b);
  DXUtil_WriteFloatRegKey(hRegKey, 'AbsoptionR', m_Absoption.r);
  DXUtil_WriteFloatRegKey(hRegKey, 'AbsoptionG', m_Absoption.g);
  DXUtil_WriteFloatRegKey(hRegKey, 'AbsoptionB', m_Absoption.b);
  DXUtil_WriteFloatRegKey(hRegKey, 'ReducedScatteringR', m_ReducedScattering.r);
  DXUtil_WriteFloatRegKey(hRegKey, 'ReducedScatteringG', m_ReducedScattering.g);
  DXUtil_WriteFloatRegKey(hRegKey, 'ReducedScatteringB', m_ReducedScattering.b);
  DXUtil_WriteFloatRegKey(hRegKey, 'RelativeIndexOfRefraction', m_fRelativeIndexOfRefraction);
  DXUtil_WriteStringRegKey(hRegKey, 'InputMesh', m_strInputMesh);
  DXUtil_WriteStringRegKey(hRegKey, 'ResultsFile', m_strResultsFile);
  DXUtil_WriteStringRegKey(hRegKey, 'InitialDir', m_strInitialDir);
  RegCloseKey(hRegKey);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
function CMyD3DApplication.ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
  adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT;
begin
  Result:= E_FAIL;

  // Debugging vertex shaders requires either REF or software vertex processing
  // and debugging pixel shaders requires REF.
  {$IFDEF DEBUG_VS}
  if (pCaps.DeviceType <> D3DDEVTYPE_REF) and
     ((dwBehavior and D3DCREATE_SOFTWARE_VERTEXPROCESSING) = 0) then Exit;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
  if (pCaps.DeviceType <> D3DDEVTYPE_REF) then Exit;
  {$ENDIF}

  // If device doesn't support 1.1 vertex shaders in HW, switch to SWVP.
  if (pCaps.VertexShaderVersion < D3DVS_VERSION(1,1)) then
  begin
    if ((dwBehavior and D3DCREATE_SOFTWARE_VERTEXPROCESSING) = 0) then Exit;
  end;

  // Need ps_2_0 for per pixel PRT
  if (pCaps.PixelShaderVersion < D3DPS_VERSION(2,0)) then Exit;

  Result:= S_OK;
end;



//-----------------------------------------------------------------------------
// Name: LaunchPRTSimulation()
// Desc: Get the settings from the user and launch the PRT simulator on another
//       thread cause it'll likely take a while and the UI would be
//       unresponsive otherwise
//-----------------------------------------------------------------------------
function CMyD3DApplication.LaunchPRTSimulation: HRESULT;
var
  dwResult: DWORD;
  nResult: Integer;
  hMenu: Windows.HMENU;
begin
  if not m_bWindowed then
  begin
    // Go back to windowed mode to display dialog boxes
    Result := ToggleFullscreen;
    if FAILED(Result) then begin Result:= DXTRACE_ERR('ToggleFullscreen', Result, UnitName); Exit; end;
  end;

  if (m_hPRTSimulationThreadId <> 0) then
  begin
    // Ask to stop the PRT simulator if it's running in the other thread
    dwResult := WaitForSingleObject(m_hPRTSimulationThreadId, 0);
    if (dwResult = WAIT_TIMEOUT) then
    begin
      nResult := MessageBox(m_hWnd, 'Simulation still processing.  Abort current simulation?',
                            'Warning', MB_YESNO or MB_ICONWARNING);
      if (nResult = IDNO) then
      begin
        Result:= S_OK;
        Exit;
      end;

      Result := StopPRTSimulation;
      if FAILED(Result) then Exit;
    end;
  end;

  m_bStopSimulator := False;
  m_bPRTSimulationComplete := False;

  Pause(True);

  // Ask the user about param settings for the PRT Simulation
  nResult := DialogBox(0, MAKEINTRESOURCE(IDD_SIMULATION_OPTIONS), m_hWnd, @StaticSimulationOptionsDlgProc);
  if (nResult = IDOK) then
  begin
    DialogBox(HInstance, MAKEINTRESOURCE(IDD_COMPRESS), m_hWnd, @StaticCompressDlgProc);

    m_bPRTSimulationFailed := False;
    m_bSHPRTReadyToRender := False;
    m_nTechnique := 1;

    // Launch the PRT simulator on another thread cause it'll
    // likely take a while and the UI would be unresponsive otherwise
    m_hPRTSimulationThreadId := CreateThread(nil, 0, @StaticPRTSimulationThreadProc,
                                             Self, 0, m_dwPRTSimulationThreadId);
    SetThreadPriority(m_hPRTSimulationThreadId, THREAD_PRIORITY_BELOW_NORMAL); //todo: Bug this to MS! (set thread priority)
  end;

  hMenu := GetSubMenu(GetMenu(m_hWnd), 1);
  if (m_hPRTSimulationThreadId <> 0)
    then EnableMenuItem(hMenu, IDM_ABORT_SIMULATION, MF_ENABLED)
    else EnableMenuItem(hMenu, IDM_ABORT_SIMULATION, MF_GRAYED);

  Pause(False);
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: StopPRTSimulation()
// Desc: Set the m_bStopSimulator and wait for the simulator to close
//-----------------------------------------------------------------------------
function CMyD3DApplication.StopPRTSimulation: HRESULT;
var
  dwResult: DWORD;
begin
  if (m_hPRTSimulationThreadId <> 0) then
  begin
    dwResult := WaitForSingleObject( m_hPRTSimulationThreadId, 0);
    if (dwResult = WAIT_TIMEOUT) then
    begin
      EnterCriticalSection(m_cs);
      m_bStopSimulator := True;
      LeaveCriticalSection(m_cs);

      // Wait for it to close
      dwResult := WaitForSingleObject(m_hPRTSimulationThreadId, 10000);
      if (dwResult = WAIT_TIMEOUT) then
      begin
        Result:= E_FAIL;
        Exit;
      end;
      m_bStopSimulator := False;
      m_hPRTSimulationThreadId := 0;
    end;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: StaticPRTSimulationThreadProc
// Desc: static helper function
//-----------------------------------------------------------------------------
function StaticPRTSimulationThreadProc(lpParameter: Pointer): DWORD; stdcall;
begin
  Result:= g_d3dApp.PRTSimulationThreadProc;
end;


//-----------------------------------------------------------------------------
// Name: PRTSimulationThreadProc()
// Desc: Load the mesh and start the simluator and save the results to a file
//-----------------------------------------------------------------------------
function CMyD3DApplication.PRTSimulationThreadProc: DWORD;
var
  pAlbedoTexture: IDirect3DTexture9;
  shMat: TD3DXSHMaterial;
begin
  // Reset precent complete
  m_fPercentDone := 0.0;

  // Cleanup any previous buffer
  SAFE_RELEASE(m_pMesh);
  SAFE_RELEASE(m_pSHPRTBuffer);

  // Load the mesh
  Result := LoadMesh(m_strInputMesh, m_pMesh);
  if FAILED(Result) then
  begin
    MessageBox(m_hWnd, 'Couldn''t load mesh.  Ensure that the mesh has texture coordinates and that they are in the [0, 1] range', 'Error', MB_OK);
    DXTRACE_ERR('LoadMesh', Result, UnitName);
    Result:= 1;
    Exit;
  end;

  // Note that the alpha value is ignored for the Diffuse, Absorption,
  // and ReducedScattering parameters of the material.
  ZeroMemory(@shMat, SizeOf(TD3DXSHMaterial));
  shMat.Diffuse := m_DiffuseReflectance;
  shMat.bMirror := False;
  shMat.bSubSurf := m_bSubsurfaceScattering;
  shMat.RelativeIndexOfRefraction := m_fRelativeIndexOfRefraction;
  shMat.Absorption := m_Absoption;
  shMat.ReducedScattering := m_ReducedScattering;

  // If the model has an albedo texture, then pass it to the simulator
  // This sample doesn't show off this functionality
  Result := D3DXSHPRTSimulationTex(m_dwOrder, m_pMesh, @shMat,
                                   m_dwNumRays, m_dwNumBounces, m_bSubsurfaceScattering,
                                   m_fLengthScale, m_bSpectral,
                                   m_pSHPRTBuffer, m_dwTextureSize, m_dwTextureSize,
                                   pAlbedoTexture, StaticSHPRTSimulatorCB);
  if FAILED(Result) then
  begin
    if (HRESULT(Result) = E_OUTOFMEMORY) then
      MessageBox(m_hWnd, 'The simulator ran out of memory.  Either reduce the texture size or increase your virtual memory.', 'Error', MB_OK );
    if (HRESULT(Result) = D3DERR_INVALIDCALL) then
      MessageBox(m_hWnd, 'The simulator failed with D3DERR_INVALIDCALL.  Check the debug output for more information about why it failed.', 'Error', MB_OK );

    m_bPRTSimulationFailed := True;

    // This returns E_FAIL if the simulation was aborted from the callback
    if (HRESULT(Result) = E_FAIL) then
    begin
      Result:= 1;
      Exit;
    end;

    DXTRACE_ERR('D3DXSHPRTSimulation', Result, UnitName);
    Result:= 1;
    Exit;
  end;

  SaveSHPRTBufferToFile(m_pSHPRTBuffer, m_strResultsFile);

  Result:= 1;
end;


//-----------------------------------------------------------------------------
// Name: SaveSHPRTBufferToFile()
// Desc: Save a SH PRT buffer to a file
//-----------------------------------------------------------------------------
function CMyD3DApplication.SaveSHPRTBufferToFile(pd3dxBuffer: ID3DXBuffer; strFile: PChar): HRESULT;
var
  hFile: THandle;
  pBuffer: Pointer;
  dwBufferSize: DWORD;
  dwWritten: DWORD;
  nResult: LongBool;
begin
  // Create a file
  hFile := CreateFile(strFile, GENERIC_WRITE, 0, nil, CREATE_ALWAYS,
                      FILE_ATTRIBUTE_NORMAL, 0);
  if (hFile = INVALID_HANDLE_VALUE) then begin Result:= DXTRACE_ERR('CreateFile', E_FAIL, UnitName); Exit; end;

  // Get the buffer pointer & size
  pBuffer      := pd3dxBuffer.GetBufferPointer;
  dwBufferSize := pd3dxBuffer.GetBufferSize;

  // Write to the file
  dwWritten := 0;
  nResult := WriteFile(hFile, pBuffer^, dwBufferSize, dwWritten, nil);
  if not nResult or (dwWritten <> dwBufferSize) then
  begin
    CloseHandle(hFile);
    Result:= DXTRACE_ERR('WriteFile', E_FAIL, UnitName);
    Exit;
  end;

  CloseHandle(hFile);
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: ReadSHPRTBufferFromFile()
// Desc: Load the previously saved SH PRT buffer from a file
//-----------------------------------------------------------------------------
function CMyD3DApplication.ReadSHPRTBufferFromFile(strFile: PChar; out ppd3dxBuffer: ID3DXBuffer): HRESULT;
var
  hFile: THandle;
  dwFileSize: DWORD;
  pBuffer: Pointer;
  dwRead: DWORD;
  nResult: LongBool;
begin
  // Open the file
  hFile := CreateFile(strFile, GENERIC_READ, FILE_SHARE_READ, nil, OPEN_EXISTING,
                      FILE_ATTRIBUTE_NORMAL, 0);
  if (hFile = INVALID_HANDLE_VALUE) then
  begin Result:= DXTRACE_ERR('CreateFile', E_FAIL); Exit; end;

  // Get the file size
  dwFileSize := GetFileSize(hFile, nil);

  // Make an ID3DXBuffer
  Result := D3DXCreateBuffer(dwFileSize, ppd3dxBuffer);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('D3DXCreateBuffer', Result, UnitName); Exit; end;

  pBuffer := ppd3dxBuffer.GetBufferPointer;

  // Fill up the ID3DXBuffer
  dwRead := 0;
  nResult := ReadFile(hFile, pBuffer^, dwFileSize, dwRead, nil);
  if not nResult or (dwRead <> dwFileSize) then 
  begin
    SAFE_RELEASE(ppd3dxBuffer);
    CloseHandle(hFile);
    Result:= DXTRACE_ERR('ReadFile', E_FAIL, UnitName);
    Exit;
  end;

  CloseHandle(hFile);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: StaticSHPRTSimulatorCB()
// Desc: static helper function
//-----------------------------------------------------------------------------
function StaticSHPRTSimulatorCB(fPercentDone: Single): HRESULT; stdcall;
begin
  Result:= g_d3dApp.SHPRTSimulatorCB(fPercentDone);
end;


//-----------------------------------------------------------------------------
// Name: SHPRTSimulatorCB()
// Desc: records the percent done and stops the simulator if requested
//-----------------------------------------------------------------------------
function CMyD3DApplication.SHPRTSimulatorCB(fPercentDone: Single): HRESULT;
begin
  // Keep only one thread accessing the shared data at a time
  EnterCriticalSection(m_cs);

  m_fPercentDone := fPercentDone;

  Result := S_OK;
  if (m_bStopSimulator) then Result := E_FAIL; // return anything except S_OK will stop the simulator

  LeaveCriticalSection(m_cs);
end;



//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Overrrides the main WndProc, so the sample can do custom message
//       handling (e.g. processing mouse, keyboard, or menu commands).
//-----------------------------------------------------------------------------
var
  s_fRemember: Single = 0.0;

function CMyD3DApplication.MsgProc(hWnd: HWND; uMsg: LongWord;
  wParam: WPARAM; lParam: LPARAM): LRESULT;
var
  iMouseX: Integer;
  iMouseY: Integer;
  hMenu: Windows.HMENU;
begin
  m_Camera.HandleMessages(hWnd, uMsg, wParam, lParam);

  case uMsg of
    WM_COMMAND:
    begin
      case LOWORD(wParam) of
        IDM_TOGGLEHELP: m_bShowHelp := not m_bShowHelp;

        IDM_RUN_SIMULATION,
        IDM_ABORT_SIMULATION:
        begin
          if (LOWORD(wParam) = IDM_RUN_SIMULATION) then
          begin
            // Ask the user about PRT simulator settings and launch
            // the PRT simulator in another thread to not block the UI thread
            LaunchPRTSimulation;
          end else
          begin
            // Stop the PRT simulator if it's running in the other thread
            Result := StopPRTSimulation;
            if FAILED(Result) then
              MessageBox(hWnd, 'PRT simulator thread failed to stop'#10, 'Error', MB_OK);
          end;

          hMenu := GetSubMenu(GetMenu(hWnd), 1);
          if (m_hPRTSimulationThreadId <> 0)
            then EnableMenuItem(hMenu, IDM_ABORT_SIMULATION, MF_ENABLED)
            else EnableMenuItem(hMenu, IDM_ABORT_SIMULATION, MF_GRAYED);
          DrawMenuBar(hWnd);
        end;

        IDM_PRT_VIEWRESULTS: DoStartupDlg;

        IDM_PRT_COMPRESSIONSETTINGS: DoCompressDlg;
      end;
    end;

    WM_KEYDOWN:
    begin
      case Char(wParam) of
        'C':
          DoCompressDlg();

        'N':
        begin
          Inc(m_nTechnique);
          m_nTechnique := m_nTechnique mod 2;

          if (m_nTechnique = 0) then  EvalLightsAndSetConstants;
        end;

        'K':
        begin
          if (not m_LightArcBall[m_nActiveLight].IsBeingDragged) then
          begin
            Inc(m_nActiveLight);
            m_nActiveLight := m_nActiveLight mod m_nNumActiveLights;
          end;
        end;

        'L':
        begin
          if (not m_LightArcBall[m_nActiveLight].IsBeingDragged) then
          begin
            m_nNumActiveLights := m_nNumActiveLights mod MAX_LIGHTS;
            Inc(m_nNumActiveLights);
            m_nActiveLight := m_nActiveLight mod m_nNumActiveLights;
            UpdateLightDir;
            EvalLightsAndSetConstants;
          end;
        end;

        'I':
        begin
          m_fLightScale := m_fLightScale * 1.25;
          EvalLightsAndSetConstants;
        end;
        'U':
        begin
          m_fLightScale := m_fLightScale * 0.8;
          EvalLightsAndSetConstants;
        end;
        'O':
        begin
          m_fLightScale := 1.0;
          EvalLightsAndSetConstants;
        end;

        'W':
        begin
          m_bWireFrame := not m_bWireFrame;
          if (m_bWireFrame)
          then m_pd3dDevice.SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME)
          else m_pd3dDevice.SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
        end;

        'E':
        begin
          m_dwNumPCAVectors := m_dwNumPCAVectors + 4;
          if (m_dwNumPCAVectors > min(m_dwOrder*m_dwOrder,MAX_PCA_VECTORS))
            then m_dwNumPCAVectors := min(m_dwOrder*m_dwOrder,MAX_PCA_VECTORS);
          CompressData;
        end;
        'R':
        begin
          m_dwNumPCAVectors := m_dwNumPCAVectors - 4;
          if (m_dwNumPCAVectors < 4) then m_dwNumPCAVectors := 4;
          CompressData;
        end;
      end;
    end;

    WM_RBUTTONDOWN:
    begin
      iMouseX := LOWORD(lParam); // GET_X_LPARAM(lParam);
      iMouseY := HIWORD(lParam); // GET_Y_LPARAM(lParam);
      m_LightArcBall[m_nActiveLight].OnBegin(iMouseX, iMouseY);
      SetCapture(hWnd);
      Result:= iTrue;
      Exit;
    end;

    WM_MOUSEMOVE:
    begin
      iMouseX := LOWORD(lParam); // GET_X_LPARAM(lParam);
      iMouseY := HIWORD(lParam); // GET_Y_LPARAM(lParam);
      if (m_LightArcBall[m_nActiveLight].IsBeingDragged)
      then m_LightArcBall[m_nActiveLight].OnMove(iMouseX, iMouseY);

      if (m_Camera.IsBeingDragged) then
      begin
        // Evaluate the lights in terms of SH coefficients
        // and set the shader constants, and call this function
        // whenever the lights or object rotates and need to be reevaluated
        UpdateLightDir;
        EvalLightsAndSetConstants;
      end;
      Result:= iTrue;
      Exit;
    end;

    WM_LBUTTONUP,
    WM_MBUTTONUP,
    WM_RBUTTONUP:
    begin
      if (uMsg = WM_RBUTTONUP) then
      begin
        m_LightArcBall[m_nActiveLight].OnEnd;
        ReleaseCapture;
      end;

      UpdateLightDir;
      EvalLightsAndSetConstants;
      Result:= iTrue;
      Exit;
    end;
  end;

  Result:= inherited MsgProc(hWnd, uMsg, wParam, lParam);
end;


//-----------------------------------------------------------------------------
// Name: UpdateLightDir()
// Desc: Make the arcball respond correctly when the camera has be rotated.
//-----------------------------------------------------------------------------
function CMyD3DApplication.UpdateLightDir: HRESULT;
var
  m, mInvView, mLastRotInv, mRot: TD3DXMatrix;
  pXBasis, pYBasis, pZBasis: PD3DXVector3;
begin
  D3DXMatrixInverse(mInvView, nil, m_mView);
  mInvView._41 := 0; mInvView._42 := 0; mInvView._43 := 0;

  D3DXMatrixInverse(mLastRotInv, nil, m_mLightRotSnapshot[m_nActiveLight]);

  mRot := m_LightArcBall[m_nActiveLight].GetRotationMatrix^;
  m_mLightRotSnapshot[m_nActiveLight] := mRot;

  // Accumulate the delta of the arcball's rotation in view space.
  // Note that per-frame delta rotations could be problematic over long periods of time.
  // m_mLightRot[m_nActiveLight] *= m_mView * mLastRotInv * mRot * mInvView;
  D3DXMatrixMultiply(m, mRot, mInvView);
  D3DXMatrixMultiply(m, mLastRotInv, m);
  D3DXMatrixMultiply(m, m_mView, m);
  D3DXMatrixMultiply(m_mLightRot[m_nActiveLight], m_mLightRot[m_nActiveLight], m);

  // Since we're accumulating delta rotations, we need to orthonormalize
  // the matrix to prevent eventual matrix skew
  pXBasis := PD3DXVector3(@m_mLightRot[m_nActiveLight]._11);
  pYBasis := PD3DXVector3(@m_mLightRot[m_nActiveLight]._21);
  pZBasis := PD3DXVector3(@m_mLightRot[m_nActiveLight]._31);
  D3DXVec3Normalize(pXBasis^, pXBasis^);
  D3DXVec3Cross(pYBasis^, pZBasis^, pXBasis^);
  D3DXVec3Normalize(pYBasis^, pYBasis^);
  D3DXVec3Cross(pZBasis^, pXBasis^, pYBasis^);

  // Transform the default direction vector by the light's rotation matrix
  D3DXVec3TransformNormal(m_vLightDir[m_nActiveLight], m_vDefaultLightDir[m_nActiveLight], m_mLightRot[m_nActiveLight]);
  D3DXVec3TransformNormal(m_vLightDir[m_nActiveLight], m_vDefaultLightDir[m_nActiveLight], m_LightArcBall[m_nActiveLight].GetRotationMatrix^);

  Result:= S_OK;
end;


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//////                                                                   //////
//////                  Parts from OptionsDlg.Cpp                        //////
//////                                                                   //////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Name: StaticSimulationOptionsDlgProc
// Desc: static helper function
//-----------------------------------------------------------------------------
function StaticStartupDlgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT {INT_PTR}; stdcall;
begin
  Result:= g_d3dApp.StartupDlgProc(hWnd, uMsg, wParam, lParam);
end;

//-----------------------------------------------------------------------------
// Name: StaticCompressDlgProc
// Desc: static helper function
//-----------------------------------------------------------------------------
function StaticCompressDlgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT {INT_PTR}; stdcall;
begin
  Result:= g_d3dApp.CompressDlgProc(hWnd, uMsg, wParam, lParam);
end;

//-----------------------------------------------------------------------------
// Name: StaticSimulationOptionsDlgProc
// Desc: static helper function
//-----------------------------------------------------------------------------
function StaticSimulationOptionsDlgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT {INT_PTR}; stdcall;
begin
  Result:= g_d3dApp.SimulationOptionsDlgProc(hWnd, uMsg, wParam, lParam);
end;


//-----------------------------------------------------------------------------
// Name: CompressDlgProc()
// Desc: Handles messages for the compress dialog
//-----------------------------------------------------------------------------
function CMyD3DApplication.CompressDlgProc(hDlg: HWND; uMsg: UINT;
  wParam: WPARAM; lParam: LPARAM): LRESULT;
var
  i: DWORD;
  hPCACombo: Windows.HWND;
  sz: array[0..255] of Char;
  nIndex: Integer;
  nSelection: Integer;
  lr: LRESULT;
  NMHDR: PNMHdr;
  nDlgId: Integer;
  pNMTDI: PNMTTDispInfo;
  hTexFormatCombo: Windows.HWND;
begin
  case uMsg of
    WM_INITDIALOG:
    begin
      g_hDlg := hDlg;

      hPCACombo := GetDlgItem(hDlg, IDC_PCA_VECTOR_COMBO);
      nIndex := 0;
      nSelection := -1;
      for i:= 1 to min(m_dwOrder*m_dwOrder,24) div 4 do //Originally: for( i=4; i<=min(m_dwOrder*m_dwOrder,24); i += 4 )
      begin
        StrLFmt(sz, 256, '%d', [i*4]);
        sz[255] := #0;
        nIndex := SendMessage(hPCACombo, CB_ADDSTRING, 0, Integer(@sz));
        if (nIndex >= 0) then
          SendMessage(hPCACombo, CB_SETITEMDATA, nIndex, (i*4));
        if (i*4 = m_dwNumPCAVectors) then nSelection := nIndex;
      end;
      if (nSelection = -1) then nSelection := nIndex;
      SendMessage(hPCACombo, CB_SETCURSEL, nSelection, 0);

      hTexFormatCombo := GetDlgItem(hDlg, IDC_TEXTURE_FORMAT_COMBO);
      nSelection := -1;
      for i:= 0 to g_dwFmt - 1 do
      begin
        StrLCopy(sz, D3DUtil_D3DFormatToString(g_fmtTexture[i], False), 256);
        sz[255] := #0;
        nIndex := SendMessage(hTexFormatCombo, CB_ADDSTRING, 0, Cardinal(@sz));
        if (nIndex >= 0) then
          SendMessage(hTexFormatCombo, CB_SETITEMDATA, nIndex, Cardinal(g_fmtTexture[i]));
        if (g_fmtTexture[i] = m_fmtTexture) then nSelection := nIndex;
      end;
      if (nSelection = -1) then nSelection := nIndex;
      SendMessage(hTexFormatCombo, CB_SETCURSEL, nSelection, 0);

      g_hToolTip := CreateWindowEx(0, TOOLTIPS_CLASS, nil, TTS_ALWAYSTIP,
                                   {$WARNINGS OFF}CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,{$WARNINGS ON}
                                   hDlg, 0, HInstance, nil);
      SendMessage(g_hToolTip, TTM_SETMAXTIPWIDTH, 0, 300);
      SendMessage(g_hToolTip, TTM_SETDELAYTIME, TTDT_AUTOPOP, 32000);
      SendMessage(g_hToolTip, TTM_SETDELAYTIME, TTDT_INITIAL, 0);
      SendMessage(g_hToolTip, TTM_SETDELAYTIME, TTDT_RESHOW, 0);
      EnumChildWindows(hDlg, @EnumChildProc, 0);
      g_hMsgProcHook := SetWindowsHookEx(WH_GETMESSAGE, GetMsgProc, HInstance, GetCurrentThreadId);

      StrLCopy(sz, '32', 256); //todo: Check sz:= '32';
      sz[255] := #0;
      SetDlgItemText(hDlg, IDC_MAX_CONSTS_EDIT, sz);

      StrLFmt(sz, 256, '%d', [m_dwTextureSize]);
      sz[255] := #0;
      SetDlgItemText(hDlg, IDC_TEXTURE_SIZE_EDIT, sz);

      UpdateConstText(hDlg, m_d3dCaps.MaxVertexShaderConst, m_dwTextureSize);

      Result:= iTrue;
      Exit;
    end;

    WM_COMMAND:
      case LOWORD(wParam) of
        IDC_PCA_VECTOR_COMBO,
        IDC_TEXTURE_FORMAT_COMBO:
          UpdateConstText(hDlg, m_d3dCaps.MaxVertexShaderConst, m_dwTextureSize);

        IDOK:
        begin
          nIndex := SendMessage(GetDlgItem(hDlg, IDC_PCA_VECTOR_COMBO), CB_GETCURSEL, 0, 0);
          lr := SendMessage(GetDlgItem(hDlg, IDC_PCA_VECTOR_COMBO), CB_GETITEMDATA, nIndex, 0);
          if (lr <> CB_ERR) then m_dwNumPCAVectors := lr;

          nIndex := SendMessage(GetDlgItem(hDlg, IDC_TEXTURE_FORMAT_COMBO), CB_GETCURSEL, 0, 0);
          lr := SendMessage(GetDlgItem(hDlg, IDC_TEXTURE_FORMAT_COMBO), CB_GETITEMDATA, nIndex, 0);
          if (lr <> CB_ERR) then m_fmtTexture := TD3DFormat(lr);

          EndDialog(hDlg, IDOK);
        end;

        IDCANCEL: EndDialog(hDlg, IDCANCEL);
      end;

    WM_NOTIFY:
    begin
      NMHDR := PNMHdr(lParam);
      if (NMHDR <> nil) and (NMHDR.code = TTN_NEEDTEXT) then
      begin
        pNMTDI := PNMTTDispInfo(lParam);
        nDlgId := GetDlgCtrlID(NMHDR.idFrom);
        GetCompressToolTipText(nDlgId, pNMTDI);
        Result:= iTrue;
        Exit;
      end;
    end;

    WM_CLOSE:
    begin
      UnhookWindowsHookEx(g_hMsgProcHook);
      DestroyWindow(g_hToolTip);
    end;
  end;

  Result:= iFalse;
end;


//-----------------------------------------------------------------------------
// Name: SimulationOptionsDlgProc()
// Desc: Handles messages for the Simulation options dialog
//-----------------------------------------------------------------------------
var
  s_bComboBoxSelChange: Boolean = False;

function CMyD3DApplication.SimulationOptionsDlgProc(hDlg: HWND; uMsg: UINT;
  wParam: WPARAM; lParam: LPARAM): LRESULT;
var
  hOrderSlider: Windows.HWND;
  hNumBouncesSpin: Windows.HWND;
  hNumRaysSpin: HWND;
  udAccel: array[0..2] of CommCtrl.UDACCEL;
  hPreDefCombo: HWND;
  i: DWORD;
  nIndex: Integer;
  sz: array[0..255] of Char;
  NMHDR: PNMHdr;
  pNMTDI: PNMTTDispInfo;
  nDlgId: Integer;
  hPCACombo: HWND;
  dwOrder: DWORD;
  bSubSurface: Boolean;
  bSpectral: Boolean;
  pMat: PPredefinedMaterials;
  lr: LRESULT;
  pLastSlash: PChar;
  szMesh: array[0..MAX_PATH-1] of Char;
  nSize: DWORD;
  pMesh: ID3DXMesh;
  hTexSizeCombo: Windows.HWND;
  nSelection: Integer;
  hr: HRESULT;
{$WRITEABLECONST ON}
const
  ofn: TOpenFilename = (
    lStructSize: SizeOf(TOpenFilename);
    lpstrFilter: '.X Files (.x)'#0'*.x'#0'All Files'#0'*.*'#0#0; nFilterIndex: 1;
    nMaxFile: 512;
    nMaxFileTitle: 512;
    lpstrTitle: 'Open Mesh File';
    Flags: OFN_FILEMUSTEXIST;
    nFileExtension: 1
  );
{$WRITEABLECONST OFF}
  HXX0001helpString = 'Texture space PRT requires a mesh with unique texture parameterization which means that every point on the surface must map to a unique point on the texture. In other words there can be no overlap in the texture space. If the mesh has texture '+'coordinates but they are not unique then the simulator will accept the mesh but the output will look incorrect. Also, the texture coordinates need to be between 0 and 1 and the texture coordinate spacing needs to be enough so that there are 2 '+'texels between pieces including the border.  Note: many mesh files in the media directory do not satisfy these requirements.';
begin
  Result:= iFalse;
  case uMsg of
    WM_INITDIALOG:
    begin
      g_hDlg := hDlg;
      SetDlgItemText(hDlg, IDC_MESH_NAME, m_strInputMesh);

      hOrderSlider := GetDlgItem(hDlg, IDC_ORDER_SLIDER);
      SendMessage(hOrderSlider, TBM_SETRANGE, 0, MAKELONG(2, 6));
      SendMessage(hOrderSlider, TBM_SETTICFREQ, 1, 0);
      SendMessage(hOrderSlider, TBM_SETPOS, 1, m_dwOrder);

      hTexSizeCombo := GetDlgItem(hDlg, IDC_TEXTURE_SIZE_COMBO);
      nSelection := -1;
      for i:= 6 to 11 do
      begin
        nSize := Trunc(Power(2.0, i));
        StrLFmt(sz, 256, '%d', [nSize]); sz[255]:= #0;
        nIndex := SendMessage(hTexSizeCombo, CB_ADDSTRING, 0, Cardinal(@sz));
        if (nIndex >= 0) then
          SendMessage(hTexSizeCombo, CB_SETITEMDATA, nIndex, nSize);
        if (nSize = m_dwTextureSize) then nSelection := nIndex;
      end;
      if (nSelection = -1) then nSelection := nIndex;
      SendMessage(hTexSizeCombo, CB_SETCURSEL, nSelection, 0);

      hNumBouncesSpin := GetDlgItem(hDlg, IDC_NUM_BOUNCES_SPIN);
      SendMessage(hNumBouncesSpin, UDM_SETRANGE, 0, MAKELONG(3, 0));
      SendMessage(hNumBouncesSpin, UDM_SETPOS, 0, MAKELONG(m_dwNumBounces, 0));

      hNumRaysSpin := GetDlgItem(hDlg, IDC_NUM_RAYS_SPIN);
      udAccel[0].nSec := 0; udAccel[0].nInc := 1;
      udAccel[1].nSec := 1; udAccel[1].nInc := 100;
      udAccel[2].nSec := 2; udAccel[2].nInc := 1000;
      SendMessage(hNumRaysSpin, UDM_SETACCEL, 3, Cardinal(@udAccel));
      SendMessage(hNumRaysSpin, UDM_SETRANGE, 0, MAKELONG(10000, 10));
      SendMessage(hNumRaysSpin, UDM_SETPOS,   0, MAKELONG(m_dwNumRays, 0));

      if m_bSubsurfaceScattering
        then CheckDlgButton(hDlg, IDC_SUBSURF_CHECK, BST_CHECKED)
        else CheckDlgButton(hDlg, IDC_SUBSURF_CHECK, BST_UNCHECKED);
      if m_bSpectral
        then CheckDlgButton(hDlg, IDC_SPECTRAL_CHECK, BST_CHECKED)
        else CheckDlgButton(hDlg, IDC_SPECTRAL_CHECK, BST_UNCHECKED);
      SendMessage(hDlg, WM_COMMAND, IDC_SUBSURF_CHECK, 0);

      hPreDefCombo := GetDlgItem(hDlg, IDC_PREDEF_COMBO);
      for i:= 0 to g_aPredefinedMaterialsSize - 1 do
      begin
        nIndex := SendMessage(hPreDefCombo, CB_ADDSTRING, 0, Cardinal(g_aPredefinedMaterials[i].strName));
        if (nIndex >= 0) then
          SendMessage(hPreDefCombo, CB_SETITEMDATA, nIndex, Cardinal(@g_aPredefinedMaterials[i]));
      end;
      SendMessage(hPreDefCombo, CB_SETCURSEL, m_dwPredefinedMatIndex, 0);

      s_bComboBoxSelChange := True;
      StrLFmt(sz, 256, '%0.2f', [m_fRelativeIndexOfRefraction]); sz[255] := #0; SetDlgItemText(hDlg, IDC_REFRACTION_EDIT, sz);
      StrLFmt(sz, 256, '%0.2f', [m_Absoption.r]);                sz[255] := #0; SetDlgItemText(hDlg, IDC_ABSORPTION_R_EDIT, sz);
      StrLFmt(sz, 256, '%0.2f', [m_Absoption.g]);                sz[255] := #0; SetDlgItemText(hDlg, IDC_ABSORPTION_G_EDIT, sz);
      StrLFmt(sz, 256, '%0.2f', [m_Absoption.b]);                sz[255] := #0; SetDlgItemText(hDlg, IDC_ABSORPTION_B_EDIT, sz);
      StrLFmt(sz, 256, '%0.2f', [m_DiffuseReflectance.r]);       sz[255] := #0; SetDlgItemText(hDlg, IDC_REFLECTANCE_R_EDIT, sz);
      StrLFmt(sz, 256, '%0.2f', [m_DiffuseReflectance.g]);       sz[255] := #0; SetDlgItemText(hDlg, IDC_REFLECTANCE_G_EDIT, sz);
      StrLFmt(sz, 256, '%0.2f', [m_DiffuseReflectance.b]);       sz[255] := #0; SetDlgItemText(hDlg, IDC_REFLECTANCE_B_EDIT, sz);
      StrLFmt(sz, 256, '%0.2f', [m_ReducedScattering.r]);        sz[255] := #0; SetDlgItemText(hDlg, IDC_SCATTERING_R_EDIT, sz);
      StrLFmt(sz, 256, '%0.2f', [m_ReducedScattering.g]);        sz[255] := #0; SetDlgItemText(hDlg, IDC_SCATTERING_G_EDIT, sz);
      StrLFmt(sz, 256, '%0.2f', [m_ReducedScattering.b]);        sz[255] := #0; SetDlgItemText(hDlg, IDC_SCATTERING_B_EDIT, sz);
      s_bComboBoxSelChange := False;

      StrLFmt(sz, 256, '%0.1f', [m_fLengthScale]); sz[255] := #0; SetDlgItemText(hDlg, IDC_LENGTH_SCALE_EDIT, sz);

      SetDlgItemText(hDlg, IDC_OUTPUT_EDIT, m_strResultsFile);

      g_hToolTip := CreateWindowEx(0, TOOLTIPS_CLASS, nil, TTS_ALWAYSTIP,
                                   {$WARNINGS OFF}CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,{$WARNINGS ON}
                                   hDlg, 0, HInstance, nil);
      SendMessage(g_hToolTip, TTM_SETMAXTIPWIDTH, 0, 300);
      SendMessage(g_hToolTip, TTM_SETDELAYTIME, TTDT_AUTOPOP, 32000);
      SendMessage(g_hToolTip, TTM_SETDELAYTIME, TTDT_INITIAL, 0);
      SendMessage(g_hToolTip, TTM_SETDELAYTIME, TTDT_RESHOW, 0);
      EnumChildWindows(hDlg, @EnumChildProc, 0);
      g_hMsgProcHook := SetWindowsHookEx(WH_GETMESSAGE, GetMsgProc, HInstance, GetCurrentThreadId);

      Result:= iTrue;
      Exit;
    end;

    WM_NOTIFY:
    begin
      NMHDR := PNMHdr(lParam);
      if (NMHDR <> nil) and (NMHDR.code = TTN_NEEDTEXT) then
      begin
        pNMTDI := PNMTTDispInfo(lParam);
        nDlgId := GetDlgCtrlID(NMHDR.idFrom);
        GetSimulationOptionsToolTipText(nDlgId, pNMTDI);
        Result:= iTrue;
        Exit;
      end;

      case wParam of
        IDC_ORDER_SLIDER:
        begin
          if (NMHDR <> nil) and (NMHDR.code = NM_RELEASEDCAPTURE) then
          begin
            hPCACombo := GetDlgItem(hDlg, IDC_PCA_VECTOR_COMBO);
            SendMessage(hPCACombo, CB_RESETCONTENT, 0, 0);
            dwOrder := SendMessage(GetDlgItem(hDlg, IDC_ORDER_SLIDER), TBM_GETPOS, 0, 0);
            nIndex := 0;
            for i:= 1 to 24 div 4 do // for( DWORD i=4; i<=24; i += 4 )
            begin
              if (i*4 > dwOrder * dwOrder) then Break;

              StrLFmt(sz, 256, '%d', [i]);
              nIndex := SendMessage(hPCACombo, CB_ADDSTRING, 0, Cardinal(@sz));
              if (nIndex >= 0) then
                SendMessage(hPCACombo, CB_SETITEMDATA, nIndex, i);
            end;
            SendMessage(hPCACombo, CB_SETCURSEL, nIndex, 0);
          end;
        end;
      end;
    end;

    WM_COMMAND:
      case LOWORD(wParam) of
        IDC_SPECTRAL_CHECK,
        IDC_SUBSURF_CHECK:
        begin
          bSubSurface := (IsDlgButtonChecked(hDlg, IDC_SUBSURF_CHECK) = BST_CHECKED);
          bSpectral   := (IsDlgButtonChecked(hDlg, IDC_SPECTRAL_CHECK) = BST_CHECKED);
          EnableWindow(GetDlgItem(hDlg, IDC_REFRACTION_TEXT), bSubSurface);
          EnableWindow(GetDlgItem(hDlg, IDC_ABSORPTION_TEXT), bSubSurface);
          EnableWindow(GetDlgItem(hDlg, IDC_SCATTERING_TEXT), bSubSurface);
          EnableWindow(GetDlgItem(hDlg, IDC_REFLECTANCE_R_EDIT), true);
          EnableWindow(GetDlgItem(hDlg, IDC_REFLECTANCE_G_EDIT), bSpectral);
          EnableWindow(GetDlgItem(hDlg, IDC_REFLECTANCE_B_EDIT), bSpectral);
          EnableWindow(GetDlgItem(hDlg, IDC_ABSORPTION_R_EDIT), bSubSurface);
          EnableWindow(GetDlgItem(hDlg, IDC_ABSORPTION_G_EDIT), bSubSurface and bSpectral);
          EnableWindow(GetDlgItem(hDlg, IDC_ABSORPTION_B_EDIT), bSubSurface and bSpectral);
          EnableWindow(GetDlgItem(hDlg, IDC_SCATTERING_R_EDIT), bSubSurface);
          EnableWindow(GetDlgItem(hDlg, IDC_SCATTERING_G_EDIT), bSubSurface and bSpectral);
          EnableWindow(GetDlgItem(hDlg, IDC_SCATTERING_B_EDIT), bSubSurface and bSpectral);
          EnableWindow(GetDlgItem(hDlg, IDC_REFRACTION_EDIT), bSubSurface);
          EnableWindow(GetDlgItem(hDlg, IDC_LENGTH_SCALE_TEXT), bSubSurface);
          EnableWindow(GetDlgItem(hDlg, IDC_LENGTH_SCALE_EDIT), bSubSurface);
        end;

        IDC_PREDEF_COMBO:
          if (HIWORD(wParam) = CBN_SELCHANGE) then
          begin
            hPreDefCombo := GetDlgItem(hDlg, IDC_PREDEF_COMBO);
            nIndex := SendMessage(hPreDefCombo, CB_GETCURSEL, 0, 0);
            lr := SendMessage(hPreDefCombo, CB_GETITEMDATA, nIndex, 0);
            if (lr <> CB_ERR) then
            begin
              // If this is the "Custom" material, don't change the numbers
              if (nIndex <> g_aPredefinedMaterialsSize-1) then
              begin
                pMat := PPredefinedMaterials(lr);
                ZeroMemory(@sz, 256);

                s_bComboBoxSelChange := True;
                StrLFmt(sz, 256, '%0.4f', [pMat.fAbsorptionCoefficient.r]); SetDlgItemText(hDlg, IDC_ABSORPTION_R_EDIT, sz);
                StrLFmt(sz, 256, '%0.4f', [pMat.fAbsorptionCoefficient.g]); SetDlgItemText(hDlg, IDC_ABSORPTION_G_EDIT, sz);
                StrLFmt(sz, 256, '%0.4f', [pMat.fAbsorptionCoefficient.b]); SetDlgItemText(hDlg, IDC_ABSORPTION_B_EDIT, sz);
                StrLFmt(sz, 256, '%0.2f', [pMat.fDiffuseReflectance.r]);    SetDlgItemText(hDlg, IDC_REFLECTANCE_R_EDIT, sz);
                StrLFmt(sz, 256, '%0.2f', [pMat.fDiffuseReflectance.g]);    SetDlgItemText(hDlg, IDC_REFLECTANCE_G_EDIT, sz);
                StrLFmt(sz, 256, '%0.2f', [pMat.fDiffuseReflectance.b]);    SetDlgItemText(hDlg, IDC_REFLECTANCE_B_EDIT, sz);
                StrLFmt(sz, 256, '%0.2f', [pMat.fScatteringCoefficient.r]); SetDlgItemText(hDlg, IDC_SCATTERING_R_EDIT, sz);
                StrLFmt(sz, 256, '%0.2f', [pMat.fScatteringCoefficient.g]); SetDlgItemText(hDlg, IDC_SCATTERING_G_EDIT, sz);
                StrLFmt(sz, 256, '%0.2f', [pMat.fScatteringCoefficient.b]); SetDlgItemText(hDlg, IDC_SCATTERING_B_EDIT, sz);
                StrLFmt(sz, 256, '%0.2f', [pMat.fRelativeIndexOfRefraction]); SetDlgItemText(hDlg, IDC_REFRACTION_EDIT, sz);
                s_bComboBoxSelChange := False;
              end;
            end;
          end;

        IDC_REFLECTANCE_R_EDIT,
        IDC_REFLECTANCE_G_EDIT,
        IDC_REFLECTANCE_B_EDIT,
        IDC_SCATTERING_R_EDIT,
        IDC_SCATTERING_G_EDIT,
        IDC_SCATTERING_B_EDIT,
        IDC_ABSORPTION_R_EDIT,
        IDC_ABSORPTION_G_EDIT,
        IDC_ABSORPTION_B_EDIT:
        begin
          if (HIWORD(wParam) = EN_CHANGE) and not s_bComboBoxSelChange then
          begin
            hPreDefCombo := GetDlgItem(hDlg, IDC_PREDEF_COMBO);
            SendMessage(hPreDefCombo, CB_SETCURSEL, g_aPredefinedMaterialsSize-1, 0);
          end;
        end;

        IDC_INPUT_BROWSE_BUTTON:
        begin
          // Warn user about unique texture parameterization requirement
          // Comment out this line if you don't want to see this warning
          MessageBox(hDlg, HXX0001helpString, // 'Texture space PRT requires a mesh with unique texture parameterization which means that every point on the surface must map to a unique point on the texture. In other words there can be no overlap in the texture space. If the mesh has texture coordinates but they are not unique then the simulator will accept the mesh but the output will look incorrect. Also, the texture coordinates need to be between 0 and 1 and the texture coordinate spacing needs to be enough so that there are 2 texels between pieces including the border.  Note: many mesh files in the media directory do not satisfy these requirements.',
                     'Texture space PRT requirements', MB_ICONWARNING or MB_OK);

          // Display the OpenFileName dialog
          ofn.lStructSize:= SizeOf(ofn);
          ofn.hWndOwner:= hDlg;
          ofn.hInstance:= 0;
          ofn.lpstrFilter:= '.X Files (.x)'#0'*.x'#0'All Files'#0'*.*'#0#0;
          ofn.lpstrCustomFilter:= nil;
          ofn.nMaxCustFilter:= 0;
          ofn.nFilterIndex:= 1;
          ofn.lpstrFile:= m_strInputMesh;
          ofn.nMaxFileTitle:= MAX_PATH;
          ofn.lpstrFileTitle:= nil;
          ofn.nMaxFileTitle:= 0;
          ofn.lpstrInitialDir:= m_strInitialDir;
          ofn.lpstrTitle:= 'Open Mesh File';
          ofn.Flags:= OFN_FILEMUSTEXIST;
          ofn.nFileOffset:= 0;
          ofn.nFileExtension:= 1;
          {$IFDEF COMPILER6_UP}
          // Handle Delphi6 case of extended OFN structure
          // (new members are invalid for Win98SE & below OS'es)
          ofn.lStructSize:= 76;

          {$ENDIF}
          (*OPENFILENAME ofn = { sizeof(OPENFILENAME), hDlg, NULL,
                              _T(".X Files (.x)\0*.x\0All Files\0*.*\0\0"),
                              NULL, 0, 1, m_strInputMesh, MAX_PATH, NULL, 0,
                              m_strInitialDir, _T("Open Mesh File"),
                              OFN_FILEMUSTEXIST, 0, 1, NULL, 0, NULL, NULL }; *)
          if GetOpenFileName(ofn) then
          begin
            StrCopy(m_strInitialDir, m_strInputMesh);
            pLastSlash := StrScan(m_strInitialDir, '\');
            if (pLastSlash <> nil) then pLastSlash^ := #0;
            SetCurrentDirectory(m_strInitialDir);
            SetDlgItemText(hDlg, IDC_MESH_NAME, m_strInputMesh);
          end;
        end;

        IDC_OUTPUT_BROWSE_BUTTON:
        begin
          // Display the OpenFileName dialog
          ofn.lStructSize:= SizeOf(ofn);
          ofn.hWndOwner:= hDlg;
          ofn.hInstance:= 0;
          ofn.lpstrFilter:= 'DAT Files (.dat)'#0'*.dat'#0'All Files'#0'*.*'#0#0;;
          ofn.lpstrCustomFilter:= nil;
          ofn.nMaxCustFilter:= 0;
          ofn.nFilterIndex:= 1;
          ofn.lpstrFile:= m_strInputMesh;
          ofn.nMaxFileTitle:= MAX_PATH;
          ofn.lpstrFileTitle:= nil;
          ofn.nMaxFileTitle:= 0;
          ofn.lpstrInitialDir:= m_strInitialDir;
          ofn.lpstrTitle:= 'Save Results File';
          ofn.Flags:= OFN_OVERWRITEPROMPT or OFN_PATHMUSTEXIST or OFN_HIDEREADONLY or OFN_NOREADONLYRETURN;
          ofn.nFileOffset:= 0;
          ofn.nFileExtension:= 1;
          ofn.lpstrDefExt := '.dat';
          {$IFDEF COMPILER6_UP}
          // Handle Delphi6 case of extended OFN structure
          // (new members are invalid for Win98SE & below OS'es)
          ofn.lStructSize:= 76;

          {$ENDIF}
          (*OPENFILENAME ofn = { sizeof(OPENFILENAME), hDlg, NULL,
                              _T("DAT Files (.dat)\0*.dat\0All Files\0*.*\0\0"),
                              NULL, 0, 1, m_strResultsFile, MAX_PATH, NULL, 0,
                              m_strInitialDir, _T("Save Results File"),
                              OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_NOREADONLYRETURN,
                              0, 1, TEXT(".dat"), 0, NULL, NULL }; *)
          if GetSaveFileName(ofn) then
          begin
            StrCopy(m_strInitialDir, m_strResultsFile);
            pLastSlash := StrScan(m_strInitialDir, '\');
            if (pLastSlash <> nil) then pLastSlash^ := #0;
            SetCurrentDirectory(m_strInitialDir);
            SetDlgItemText(hDlg, IDC_OUTPUT_EDIT, m_strResultsFile);
          end;
        end;

        IDOK:
        begin
          GetDlgItemText(hDlg, IDC_MESH_NAME, m_strInputMesh, MAX_PATH);

          DXUtil_FindMediaFileCb(szMesh, SizeOf(szMesh), m_strInputMesh);
          if (GetFileAttributes(szMesh) = $FFFFFFFF) then
          begin
            MessageBox(m_hWnd, 'Couldn''t find the mesh file.  Be sure the mesh file exists.', 'Error', MB_OK);
            Exit;
          end;

          hr := D3DXLoadMeshFromX(szMesh, D3DXMESH_MANAGED, m_pd3dDevice,
                                  nil, nil, nil, nil, pMesh);
          if FAILED(hr) then
          begin
            SAFE_RELEASE(pMesh);
            MessageBox(m_hWnd, 'Couldn''t load the mesh file.  Be sure the mesh file is valid.', 'Error', MB_OK);
            Exit;
          end;

          // Make sure there are texture coordinates which are required for texturing
          if (((pMesh.GetFVF and D3DFVF_TEXCOUNT_MASK) shr D3DFVF_TEXCOUNT_SHIFT) = 0) then
          begin
            SAFE_RELEASE(pMesh);
            MessageBox(m_hWnd, 'The mesh file does not have texture coordinates.  They are needed for per pixel PRT.', 'Error', MB_OK);
            Exit;
          end;
          SAFE_RELEASE(pMesh);

          m_dwOrder := SendMessage(GetDlgItem(hDlg, IDC_ORDER_SLIDER), TBM_GETPOS, 0, 0);
          m_dwNumBounces := SendMessage(GetDlgItem(hDlg, IDC_NUM_BOUNCES_SPIN), UDM_GETPOS, 0, 0);
          m_dwNumRays := SendMessage(GetDlgItem(hDlg, IDC_NUM_RAYS_SPIN), UDM_GETPOS, 0, 0);
          m_bSubsurfaceScattering := (IsDlgButtonChecked(hDlg, IDC_SUBSURF_CHECK) = BST_CHECKED);
          m_bSpectral := (IsDlgButtonChecked(hDlg, IDC_SPECTRAL_CHECK) = BST_CHECKED);
          m_dwPredefinedMatIndex := SendMessage(GetDlgItem(hDlg, IDC_PREDEF_COMBO), CB_GETCURSEL, 0, 0);

          nIndex := SendMessage(GetDlgItem(hDlg, IDC_TEXTURE_SIZE_COMBO), CB_GETCURSEL, 0, 0);
          lr := SendMessage(GetDlgItem(hDlg, IDC_TEXTURE_SIZE_COMBO), CB_GETITEMDATA, nIndex, 0);
          if (lr <> CB_ERR) then m_dwTextureSize := lr;

          try
            GetDlgItemText(hDlg, IDC_REFRACTION_EDIT, sz, 256);    m_fRelativeIndexOfRefraction:= StrToFloat(sz);
            GetDlgItemText(hDlg, IDC_ABSORPTION_R_EDIT, sz, 256);  m_Absoption.r:= StrToFloat(sz);
            GetDlgItemText(hDlg, IDC_ABSORPTION_G_EDIT, sz, 256);  m_Absoption.g:= StrToFloat(sz);
            GetDlgItemText(hDlg, IDC_ABSORPTION_B_EDIT, sz, 256);  m_Absoption.b:= StrToFloat(sz);
            GetDlgItemText(hDlg, IDC_REFLECTANCE_R_EDIT, sz, 256); m_DiffuseReflectance.r:= StrToFloat(sz);
            GetDlgItemText(hDlg, IDC_REFLECTANCE_G_EDIT, sz, 256); m_DiffuseReflectance.g:= StrToFloat(sz);
            GetDlgItemText(hDlg, IDC_REFLECTANCE_B_EDIT, sz, 256); m_DiffuseReflectance.b:= StrToFloat(sz);
            GetDlgItemText(hDlg, IDC_SCATTERING_R_EDIT, sz, 256);  m_ReducedScattering.r:= StrToFloat(sz);
            GetDlgItemText(hDlg, IDC_SCATTERING_G_EDIT, sz, 256);  m_ReducedScattering.g:= StrToFloat(sz);
            GetDlgItemText(hDlg, IDC_SCATTERING_B_EDIT, sz, 256);  m_ReducedScattering.b:= StrToFloat(sz);
            GetDlgItemText(hDlg, IDC_LENGTH_SCALE_EDIT, sz, 256);  m_fLengthScale:= StrToFloat(sz);
          except
            Result:= iFalse;
            Exit;
          end;

          GetDlgItemText(hDlg, IDC_OUTPUT_EDIT, m_strResultsFile, MAX_PATH);

          EndDialog(hDlg, IDOK);
        end;

        IDCANCEL:
          EndDialog(hDlg, IDCANCEL);
      end;

    WM_CLOSE:
    begin
      UnhookWindowsHookEx(g_hMsgProcHook);
      DestroyWindow(g_hToolTip);
    end;
  end;

  Result:= iFalse;
end;


//-----------------------------------------------------------------------------
// Name: StartupDlgProc()
// Desc: Handles messages for the startup dialog
//-----------------------------------------------------------------------------
function CMyD3DApplication.StartupDlgProc(hDlg: HWND; uMsg: UINT;
  wParam: WPARAM; lParam: LPARAM): LRESULT;
var
  NMHDR: PNMHdr;
  pNMTDI: PNMTTDispInfo;
//  ofn: TOpenFilename;
  nDlgId: Integer;
  pLastSlash: PChar;
  szMesh: array[0..MAX_PATH-1] of Char;
  pMesh: ID3DXMesh;
{$WRITEABLECONST ON}
const
  ofn: TOpenFilename = (
    lStructSize: SizeOf(TOpenFilename);
    lpstrFilter: '.X Files (.x)'#0'*.x'#0'All Files'#0'*.*'#0#0; nFilterIndex: 1;
    nMaxFile: 512;
    nMaxFileTitle: 512;
    lpstrTitle: 'Open Mesh File';
    Flags: OFN_FILEMUSTEXIST;
    nFileExtension: 1
  );
{$WRITEABLECONST OFF}
begin
  case uMsg of
    WM_INITDIALOG:
    begin
      g_hDlg := hDlg;

      SetDlgItemText(hDlg, IDC_INPUT_MESH_EDIT, m_strInputMesh);
      SetDlgItemText(hDlg, IDC_SIM_RESULTS_EDIT, m_strResultsFile);

      g_hToolTip := CreateWindowEx(0, TOOLTIPS_CLASS, nil, TTS_ALWAYSTIP,
                                   {$WARNINGS OFF}CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,{$WARNINGS ON}
                                   hDlg, 0, HInstance, nil);
      SendMessage(g_hToolTip, TTM_SETMAXTIPWIDTH, 0, 300);
      SendMessage(g_hToolTip, TTM_SETDELAYTIME, TTDT_AUTOPOP, 32000);
      SendMessage(g_hToolTip, TTM_SETDELAYTIME, TTDT_INITIAL, 0);
      SendMessage(g_hToolTip, TTM_SETDELAYTIME, TTDT_RESHOW, 0);
      EnumChildWindows(hDlg, @EnumChildProc, 0);
      g_hMsgProcHook := SetWindowsHookEx(WH_GETMESSAGE, GetMsgProc, HInstance, GetCurrentThreadId);

      Result:= iTrue;
      Exit;
    end;

    WM_NOTIFY:
    begin
      NMHDR := PNMHdr(lParam);
      if (NMHDR <> nil) and (NMHDR.code = TTN_NEEDTEXT) then
      begin
        pNMTDI := PNMTTDispInfo(lParam);
        nDlgId := GetDlgCtrlID(NMHDR.idFrom);
        GetStartupToolTipText(nDlgId, pNMTDI);
        Result:= iTrue;
        Exit;
      end;
    end;

    WM_COMMAND:
      case LOWORD(wParam) of
        IDC_RUN_PRT_SIM: EndDialog(hDlg, IDC_RUN_PRT_SIM);

        IDC_INPUT_MESH_BROWSE_BUTTON:
        begin
//          FillChar(ofn, SizeOf(ofn), #0);
          // Display the OpenFileName dialog
          ofn.lStructSize:= SizeOf(ofn);
          ofn.hWndOwner:= hDlg;
          ofn.hInstance:= 0;
          ofn.lpstrFilter:= '.X Files (.x)'#0'*.x'#0'All Files'#0'*.*'#0#0;
          ofn.lpstrCustomFilter:= nil;
          ofn.nMaxCustFilter:= 0;
          ofn.nFilterIndex:= 1;
          ofn.lpstrFile:= m_strInputMesh;
          ofn.nMaxFileTitle:= MAX_PATH;
          ofn.lpstrFileTitle:= nil;
          ofn.nMaxFileTitle:= 0;
          ofn.lpstrInitialDir:= m_strInitialDir;
          ofn.lpstrTitle:= 'Open Mesh File';
          ofn.Flags:= OFN_FILEMUSTEXIST;
          ofn.nFileOffset:= 0;
          ofn.nFileExtension:= 1;
          {$IFDEF COMPILER6_UP}
          // Handle Delphi6 case of extended OFN structure
          // (new members are invalid for Win98SE & below OS'es)
          ofn.lStructSize:= 76;

          {$ENDIF}
          (*OPENFILENAME ofn = { sizeof(OPENFILENAME), hDlg, NULL,
                              _T(".X Files (.x)\0*.x\0All Files\0*.*\0\0"),
                              NULL, 0, 1, m_strInputMesh, MAX_PATH, NULL, 0,
                              m_strInitialDir, _T("Open Mesh File"),
                              OFN_FILEMUSTEXIST, 0, 1, NULL, 0, NULL, NULL }; *)
          if GetOpenFileName(ofn) then
          begin
            StrCopy(m_strInitialDir, m_strInputMesh);
            pLastSlash := StrScan(m_strInitialDir, '\');
            if (pLastSlash <> nil) then pLastSlash^ := #0;
            SetCurrentDirectory(m_strInitialDir);
            SetDlgItemText(hDlg, IDC_INPUT_MESH_EDIT, m_strInputMesh);
          end;
        end;

        IDC_SIM_RESULTS_BROWSE_BUTTON:
        begin
//          FillChar(ofn, SizeOf(ofn), #0);
          // Display the OpenFileName dialog
          ofn.lStructSize:= SizeOf(ofn);
          ofn.hWndOwner:= hDlg;
          ofn.hInstance:= 0;
          ofn.lpstrFilter:= 'DAT Files (.dat)'#0'*.dat'#0'All Files'#0'*.*'#0#0;
          ofn.lpstrCustomFilter:= nil;
          ofn.nMaxCustFilter:= 0;
          ofn.nFilterIndex:= 1;
          ofn.lpstrFile:= m_strResultsFile;
          ofn.nMaxFileTitle:= MAX_PATH;
          ofn.lpstrFileTitle:= nil;
          ofn.nMaxFileTitle:= 0;
          ofn.lpstrInitialDir:= m_strInitialDir;
          ofn.lpstrTitle:= 'Open Results File';
          ofn.Flags:= OFN_FILEMUSTEXIST;
          ofn.nFileOffset:= 0;
          ofn.nFileExtension:= 1;
          {$IFDEF COMPILER6_UP}
          // Handle Delphi6 case of extended OFN structure
          // (new members are invalid for Win98SE & below OS'es)
          ofn.lStructSize:= 76;

          {$ENDIF}
(*          OPENFILENAME ofn = { sizeof(OPENFILENAME), hDlg, NULL,
                              _T("DAT Files (.dat)\0*.dat\0All Files\0*.*\0\0"),
                              NULL, 0, 1, m_strResultsFile, MAX_PATH, NULL, 0,
                              m_strInitialDir, _T("Open Results File"),
                              OFN_FILEMUSTEXIST, 0, 1, NULL, 0, NULL, NULL }; *)
          if GetOpenFileName(ofn) then
          begin
            StrCopy(m_strInitialDir, m_strResultsFile);
            pLastSlash := StrScan(m_strInitialDir, '\');
            if (pLastSlash <> nil) then pLastSlash^ := #0;
            SetCurrentDirectory(m_strInitialDir);
            SetDlgItemText(hDlg, IDC_SIM_RESULTS_EDIT, m_strResultsFile);
          end;
        end;

        IDC_VIEW_RESULTS:
        begin
          GetDlgItemText(hDlg, IDC_INPUT_MESH_EDIT, m_strInputMesh, MAX_PATH);
          GetDlgItemText(hDlg, IDC_SIM_RESULTS_EDIT, m_strResultsFile, MAX_PATH);

          if (GetFileAttributes(m_strResultsFile) = $FFFFFFFF) then
          begin
            MessageBox(m_hWnd, 'Couldn''t find the simulator results file.  Run the simulator first to precompute the transfer vectors for the mesh.', 'Error', MB_OK);
          end else
          begin
            DXUtil_FindMediaFileCb(szMesh, SizeOf(szMesh), m_strInputMesh);
            if (GetFileAttributes(szMesh) = $FFFFFFFF) then
            begin
              MessageBox(m_hWnd, 'Couldn''t find the mesh file.  Be sure the mesh file exists.', 'Error', MB_OK);
            end else
            begin
              Result := D3DXLoadMeshFromX(szMesh, D3DXMESH_MANAGED, m_pd3dDevice,
                                          nil, nil, nil, nil, pMesh);
              if FAILED(Result) then
              begin
                SAFE_RELEASE(pMesh);
                MessageBox(m_hWnd, 'Couldn''t load the mesh file.  Be sure the mesh file is valid.', 'Error', MB_OK);
              end else
              begin
                // Make sure there are texture coordinates which are required for texturing
                if (((pMesh.GetFVF and D3DFVF_TEXCOUNT_MASK) shr D3DFVF_TEXCOUNT_SHIFT) = 0) then
                begin
                  SAFE_RELEASE(pMesh);
                  MessageBox(m_hWnd, 'The mesh file does not have texture coordinates.  They are needed for per pixel PRT.', 'Error', MB_OK);
                end else
                begin
                  SAFE_RELEASE(pMesh);

                  EndDialog(hDlg, IDC_VIEW_RESULTS);
                end;
              end;
            end;
          end;
        end;

        IDCANCEL: EndDialog(hDlg, IDCANCEL);
      end;

    WM_CLOSE:
    begin
      UnhookWindowsHookEx(g_hMsgProcHook);
      DestroyWindow(g_hToolTip);
    end;
  end;

  Result:= iFalse;
end;

end.

